/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-08 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus;

import java.util.ArrayList;


/**
 * Keeps track of the number of columns, their size, and the gutter size. Does no implementation.
 * For implementation, see the appropriate Column class.
 * 
 * @author alb
 */
public class Columns
{
    private int lineNumber;
    private ArrayList<Column> columns;
    private GDD gdd = GDD.getInstance();

    public Columns()
    {
        this( PlatyConst.DEFAULT_NUMBER_OF_COLUMNS );
    }
    
    public Columns( int howMany ) // is this used?
    {
        createColumns( howMany, 0f );
    }

    public Columns( int howMany, final float verticalSkip )
    {
        createColumns( howMany, verticalSkip );
    }
    /**
     * Sets the number of columns. For the time being, these are equally wide and spaced
     * columns. Eventually, we'll permit irregular shapes and varying widths.
     * @param newCount number of new columns
     * @return number of created columns
     */
    public int createColumns( final int newCount, final float verticalSkip )
    {
        /** the number of columns to implement. Must be > 0 */
        final int cols = ( newCount < 1 ? 1 : newCount ); //TODO: error msg if < 1

        /** the width of the space into which the columns + gutters must fit */
        float textWidth = GDD.getInstance().getPageSize().getWidth() -
                          GDD.getInstance().getMargin().getSize( Margins.LEFT ) -
                          GDD.getInstance().getMargin().getSize( Margins.RIGHT );

        /** number of gutters. There is no gutter to the right of the rightmost column  */
        int numberOfGutters = cols - 1;

        /** the amount of space used by the gutters */
        float gutterSpace = numberOfGutters * getRecommendedGutterSize( cols );

        /** width of each column */
        float colWidth = ( textWidth - gutterSpace ) / cols;

        // now add the columns
        this.columns = new ArrayList<Column>();

        Column c;

        for( int i = 0; i < cols; i++ )
        {
            // if at the last column, margin is 0
            if ( i + 1 == cols ) {
                c = new Column( colWidth, 0f, verticalSkip );
            }
            else {
                c = new Column( colWidth, getRecommendedGutterSize( cols ), verticalSkip );
            }

            columns.add( c );
        }

        lineNumber = GID.getInstance().getCurrLine().getLineNumber();
        return( getCount() );
    }

    /**
     * If we changed column count in the middle of a page, then the columns at the bottom
     * half of the page will be partial height. However, when we start a succeeding page, those
     * columns should now start at the top of the text area of the new page. This function handles
     * that change by reducing verticalSkip to 0 and adjusting height to be the full height of the
     * text area.
     * @return  how many columns were modified.
     */
    public int startColumnsAtTop()
    {
        int i;
        Column col;

        for( i = 0; i < columns.size(); i++ ) {
            col = columns.get( i );
            columns.remove( i );
            col.setHeight( gdd.getPageSize().getHeight() -
                           gdd.getMargin().getSize( Margins.TOP ) -
                           gdd.getMargin().getSize( Margins.BOTTOM ));
            col.setVertSkip( 0f );
            columns.add( i, col );
        }
        return( i );
    }

    /**
     * Return a designated column
     * @param whichCol the index of the column to get ( 0 = leftmost )
     * @return the specified column, or null on error
     */
    public Column getColumn( final int whichCol )
    {
        if ( whichCol < 0 || whichCol > ( columns.size() - 1 )) {
            return( null );
        }
        else {
            return( columns.get( whichCol ));
        }
    }

    public int getCount()
    {
        return( columns.size() );
    }

    /**
     * Figure out the recommended gutter size for regular columns
     * @param numberOfColumns
     * @return recommended size of gutter in points, or -1f in case of 0 or less columns.
     */
    private float getRecommendedGutterSize( final int numberOfColumns )
    {
        if ( numberOfColumns > 0 ) {
            switch ( numberOfColumns )
            {
                case 1: return(  0f );
                case 2: return( 10f );
                case 3: return(  8f );
                case 4: return(  8f );
                default: return( 6f );
            }
        }
        else {
           return( Status.ERR_NUMBER_OF_COLUMNS );
        }
    }

    public int getLineNumber()
    {
        return( lineNumber );
    }

    public String dump()
    {
        final Literals lits = Literals.getInstance();

        final StringBuffer sb = new StringBuffer();
        sb.append( lits.getLit( "NUMBER_OF_COLUMNS" ))
          .append( " (" )
          .append( lits.getLit( "LINE" ))
          .append( " " )
          .append( lineNumber )
          .append(  ") " )
          .append( columns.size() );
        return( sb.toString() );
    }
}
