/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-07 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus;

import platypus.commands.CommandParseMacro;

/**
 * Abstract Class
 *
 * Converts command strings into actions. Mostly, it's a look up into a command table, keying off
 * the command in the document. When the command object is found, its process() method is called,
 * passing in the command as found in the input document. This is done for all commands.
 * 
 * @author alb
 */
public abstract class CommandProcessor
{
    /** instance of the GDD singleton */
    protected GDD gdd = null;

    /** instance of the GID singleton */
    protected  GID gid;

    /** instance of the Literals singleton */
    protected  Literals lits;

    /**
     * get the instances of the singletons we use here.
     */
    public CommandProcessor()
    {
        // cannot init GDD here, because Command is created withing GDD, so this would be = null;
        gid = GID.getInstance();
        lits = Literals.getInstance();
    }

    /**
     * The main switch that identifies the commands and dispatches them to the correct routine
     * @param tok the command string (with surrounding brackets)
     * @return OK or the status from the command. If command invalid, returns Status.INVALID_TOKEN
     */
    public int process( final char[] tok )
    {
        if ( tok == null ) {
            return( Status.INVALID_TOKEN_NULL );
        }

        if ( tok.length < PlatyConst.MIN_TOKEN_LEN ) {
            return( Status.INVALID_TOKEN );
        }

        if ( gdd == null ) {
            gdd = GDD.getInstance();      
        }

        /** our return value, as some command operations do return values */
        int status;

        final String tokStr = new String( tok );     // Don't use toString() for this.
        final String commandStr;

        // if the command is a macro substitution, make the substitution and put it in
        // the token queue.
        if ( tokStr.startsWith( "[$" )) {
            return ( CommandParseMacro.process( gdd, gid, lits, tokStr ));
        }

        // if the command is a 'print macro' command, the [* is the root, so recognize this.
        if ( tokStr.startsWith( "[*" )) {
            commandStr = "[*";
        }
        else {
            // otherwise, if the command contains a parameter (signalled by presence of a : ), then
            // search for the command root, which goes from the [ up to the : inclusive of both.
            final int colonLocation = tokStr.indexOf( ':' );
            if ( colonLocation >= 0 ) {
                // if function has a parameter, grab the command root
                commandStr = tokStr.substring( 0, colonLocation + 1 );
            }
            else {
                commandStr = tokStr;
            }
        }

        Command thisCommand = gdd.getCommandTable().get( commandStr );
        if ( thisCommand != null ) {
            status = thisCommand.process( tokStr );
        }
        else {
            // if the command lookup failed
            GDD.getInstance().getLogger().warning( lits.getLit( "ERR_INVALID_COMMAND" ) +
                    " " + tokStr +  " " + lits.getLit( "AT_LINE" ) + ": " +
                    gid.getCurrLine().getLineNumber() );
            status = Status.INVALID_COMMAND;
        }
        return( status );
    }

    /**
     * prints a warning that a particular command is not supported
     * in the current output file format.
     *
     * This message is the default behavior in this superclass, so for all
     * methods that are typically handled in subclasses. This way, if one
     * of the commands is not implemented in a subclass (such as in HTML,
     * for example), the default is to revert to this warning.
     *
     * As a result, subclasses need only implement the commands they support
     * and not worry about generating error messages for Platypus commands
     * they don't implement.
     *
     * @param commandDescription String describing the unsupported command
     */
    private void notSupportedWarning( final String commandDescription )
    {
        String outputFormat = Platypus.platyStrings.getString( "$_outputFormat" );
        if ( outputFormat == null ) {
            outputFormat = lits.getLit( "UNDEFINED_OUTPUT_FORMAT" );
        }

        gdd.getLogger().warning(
                lits.getLit( "LINE_NUM_SIGN" ) +
                gid.getCurrLine().getLineNumber() + " " +
                lits.getLit( commandDescription ) + " " +
                lits.getLit( "IS_NOT_SUPPORTED_IN" ) + " " +
                outputFormat  + ". " +
                lits.getLit( "IGNORED" ));
    }
}
