/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-08 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus;

import java.util.TreeMap;

import platypus.commands.CommandDefineMacro;
import platypus.commands.CommandDoNothing;
import platypus.commands.CommandPrintMacro;

/**
 * All supported commands are placed in a tree, where each node consists of the command
 * class, which contains the method(s) to execute. The default commands loaded here,
 * all contain the same method, which states that the command is not supported for that
 * particular output file format. The individual plugins for the output formats (PDF, etc.)
 * overwrite these Command classes with their own.
 *
 * @author alb
 */
public class CommandTable
{
    /** a tree consisting of the command lexemes and the associated commands */
    private final TreeMap<String, Command> commands;

    private final Literals lits = Literals.getInstance();

    public CommandTable()
    {
        commands = new TreeMap<String, Command>();
        loadCommands();
    }

    /**
     * Load a single command into the command tree
     * @param com  the command class, indexed by the string that Platypus uses to identify it.
     */
    public void load( Command com )
    {
        commands.put( com.getAbbrev(), com );
    }

    /**
     * Loads all the supported commands into the CommandID tree. These all default to
     * the same action: output of a message that the command is not supported for this
     * file type.
     *
     * Individual plug-ins for file formats load their commands into the table, thereby
     * overriding these defaults.
     */
    private void loadCommands()
    {
        loadCorePlatypusCommands();

        // load commands that are overwritten by output file plug-in,
        loadDefaultCommands();
        loadDefaultSpecialChars();
    }

    /**
     * Load commands that are used within the core Platypus operations. These are constant
     * across all file formats, and so they are not overwritten by plug-ins.
     */
    private void loadCorePlatypusCommands()
    {
        load ( new CommandDefineMacro() );      // [def: ...
        load ( new CommandDoNothing() );        // [donothing]
        load ( new CommandPrintMacro() );       // [* ...
    }

    /**
     * Load the commands that do all actions except output symbols and special characters. These
     * commands should be overwritten by the plug-in for the output file.
     */
    private void loadDefaultCommands()
    {
        load( new Command( "[]",              lits.getLit( "NEW_PARAGRAPH"),TokTyp.NEW_PARAGRAPH ));
        load( new Command( "[+b]",            lits.getLit( "BOLD_ON"      ),TokTyp.BOLD_ON       ));
        load( new Command( "[+i]",            lits.getLit( "ITALICS_ON"   ),TokTyp.ITALICS_ON    ));
        load( new Command( "[+marginmirror]", lits.getLit( "MARGINS_MIRRORED" ),
                                                                            TokTyp.MARGIN_MIRROR ));
        load( new Command( "[+st]",           lits.getLit( "STRIKETHRU_ON"),TokTyp.STRIKETHRU_ON ));
        load( new Command( "[+u]",            lits.getLit( "UNDERLINE_ON" ),TokTyp.UNDERLINE_ON  ));
        load( new Command( "[-b]",            lits.getLit( "BOLD_OFF"     ),TokTyp.BOLD_OFF      ));
        load( new Command( "[-i]",            lits.getLit( "ITALICS_OFF"  ),TokTyp.ITALICS_OFF   ));
        load( new Command( "[-st]",           lits.getLit( "STRIKETHRU_OFF" ),
                                                                            TokTyp.STRIKETHRU_OFF));
        load( new Command( "[-u]",            lits.getLit( "UNDERLINE_OFF"),TokTyp.UNDERLINE_OFF ));
        load( new Command( "[align:",         lits.getLit( "ALIGNMENT"    ),TokTyp.ALIGNMENT     ));
        load( new Command( "[bmargin:",       lits.getLit( "BOTTOM_MARGIN"),TokTyp.BOTTOM_MARGIN ));
        load( new Command( "[columns:",       lits.getLit( "DO_COLUMNS"   ),TokTyp.COLUMNS       ));
        load( new Command( "[dump:",          lits.getLit( "DUMP"         ),TokTyp.DUMP          ));
        load( new Command( "[ff:",            lits.getLit( "DO_FONT_FACE" ),TokTyp.FONT_FACE     ));
        load( new Command( "[fsize:",         lits.getLit( "DO_FONT_SIZE" ),TokTyp.FONT_SIZE     ));
        load( new Command( "[leading:",       lits.getLit( "DO_LEADING"   ),TokTyp.LEADING       ));
        load( new Command( "[lmargin:",       lits.getLit( "LEFT_MARGIN"  ),TokTyp.LEFT_MARGIN   ));
        load( new Command( "[noindent]",      lits.getLit( "NO_INDENT"    ),TokTyp.NO_INDENT     ));
        load( new Command( "[pageheight:",    lits.getLit( "PAGE_HEIGHT"  ),TokTyp.PAGE_HEIGHT   ));
        load( new Command( "[pagenumber:",    lits.getLit( "PAGE_NUMBER"  ),TokTyp.PAGE_NUMBER   ));
        load( new Command( "[pagenumberformat:",
                                              lits.getLit( "PAGE_NUMBER_FORMAT" ),
                                                                       TokTyp.PAGE_NUMBER_FORMAT ));
        load( new Command( "[pagesize:",      lits.getLit( "PAGE_SIZE"    ),TokTyp.PAGE_SIZE     ));
        load( new Command( "[pagewidth:",     lits.getLit( "PAGE_WIDTH"   ),TokTyp.PAGE_WIDTH    ));
        load( new Command( "[paraindent:",    lits.getLit( "PARA_INDENT_1ST_LINE"  ),
                                                                            TokTyp.PARA_INDENT_1 ));
        load( new Command( "[paraskip:",      lits.getLit( "PARASKIP"     ),TokTyp.PARA_SKIP     ));
        load( new Command( "[pg]",            lits.getLit( "NEW_PAGE"     ),TokTyp.NEW_PAGE      ));
        load( new Command( "[rmargin:",       lits.getLit( "RIGHT_MARGIN" ),TokTyp.RIGHT_MARGIN  ));
        load( new Command( "[tmargin:",       lits.getLit( "TOP_MARGIN"   ),TokTyp.TOP_MARGIN    ));
    }

    /**
     * Load commands to ouptut symbols and foreign characters. These should be overwritten by
     * the plug-in for the output file.
     */
    private void loadDefaultSpecialChars()
    {
        load( new Command( "[!^]",      lits.getLit( "INVERTED_EXCLAM" ),TokTyp.INVERTED_EXCLAM   ));
        load( new Command( "[?^]",      lits.getLit( "INVERTED_QUESTION_MARK" ),
                                                                        TokTyp.INVERTED_QUESTION ));

        load( new Command( "[a']",      lits.getLit( "A_ACUTE" ),       TokTyp.A_ACUTE           ));
        load( new Command( "[a^]",      lits.getLit( "A_CIRCUMFLEX" ),  TokTyp.A_CIRCUMFLEX      ));
        load( new Command( "[a`]",      lits.getLit( "A_GRAVE" ),       TokTyp.A_GRAVE           ));
        load( new Command( "[a\"]",     lits.getLit( "A_UMLAUT" ),      TokTyp.A_UMLAUT          ));
        load( new Command( "[A']",      lits.getLit( "A_CAP_ACUTE" ),   TokTyp.A_CAP_ACUTE       ));
        load( new Command( "[A^]",      lits.getLit( "A_CAP_CIRCUMFLEX" ),TokTyp.A_CAP_CIRCUMFLEX));
        load( new Command( "[A`]",      lits.getLit( "A_CAP_GRAVE" ),   TokTyp.A_CAP_GRAVE       ));
        load( new Command( "[A\"]",     lits.getLit( "A_CAP_UMLAUT" ),  TokTyp.A_CAP_UMLAUT      ));
        load( new Command( "[c_]",      lits.getLit( "C_CEDILLA" ),     TokTyp.C_CEDILLA         ));
        load( new Command( "[C_]",      lits.getLit( "C_CAP_CEDILLA" ), TokTyp.C_CAP_CEDILLA     ));
        load( new Command( "[copyright]",lits.getLit( "COPYRIGHT" ),    TokTyp.COPYRIGHT         ));
        load( new Command( "[e']",      lits.getLit( "E_ACUTE" ),       TokTyp.E_ACUTE           ));
        load( new Command( "[e^]",      lits.getLit( "E_CIRCUMFLEX" ),  TokTyp.E_CIRCUMFLEX      ));
        load( new Command( "[e`]",      lits.getLit( "E_GRAVE" ),       TokTyp.E_GRAVE           ));
        load( new Command( "[e\"]",     lits.getLit( "E_UMLAUT" ),      TokTyp.E_UMLAUT          ));
        load( new Command( "[E']",      lits.getLit( "E_CAP_ACUTE" ),   TokTyp.E_CAP_ACUTE       ));
        load( new Command( "[E^]",      lits.getLit( "E_CAP_CIRCUMFLEX" ),TokTyp.E_CAP_CIRCUMFLEX));
        load( new Command( "[E`]",      lits.getLit( "E_CAP_GRAVE" ),   TokTyp.E_CAP_GRAVE       ));
        load( new Command( "[E\"]",     lits.getLit( "E_CAP_UMLAUT" ),  TokTyp.E_CAP_UMLAUT      ));
        load( new Command( "[i']",      lits.getLit( "I_ACUTE" ),       TokTyp.I_ACUTE           ));
        load( new Command( "[i^]",      lits.getLit( "I_CIRCUMFLEX" ),  TokTyp.I_CIRCUMFLEX      ));
        load( new Command( "[i`]",      lits.getLit( "I_GRAVE" ),       TokTyp.I_GRAVE           ));
        load( new Command( "[i\"]",     lits.getLit( "I_UMLAUT" ),      TokTyp.I_UMLAUT          ));
        load( new Command( "[I']",      lits.getLit( "I_CAP_ACUTE" ),   TokTyp.I_CAP_ACUTE       ));
        load( new Command( "[I^]",      lits.getLit( "I_CAP_CIRCUMFLEX" ),TokTyp.I_CAP_CIRCUMFLEX));
        load( new Command( "[I`]",      lits.getLit( "I_CAP_GRAVE" ),   TokTyp.I_CAP_GRAVE       ));
        load( new Command( "[I\"]",     lits.getLit( "I_CAP_UMLAUT" ),  TokTyp.I_CAP_UMLAUT      ));
        load( new Command( "[n~]",      lits.getLit( "N_TILDE" ),       TokTyp.N_TILDE           ));
        load( new Command( "[N~]",      lits.getLit( "N_CAP_TILDE" ),   TokTyp.N_CAP_TILDE       ));
        load( new Command( "[o']",      lits.getLit( "O_ACUTE" ),       TokTyp.O_ACUTE           ));
        load( new Command( "[o^]",      lits.getLit( "O_CIRCUMFLEX" ),  TokTyp.O_CIRCUMFLEX      ));
        load( new Command( "[o`]",      lits.getLit( "O_GRAVE" ),       TokTyp.O_GRAVE           ));
        load( new Command( "[o\"]",     lits.getLit( "O_UMLAUT" ),      TokTyp.O_UMLAUT          ));
        load( new Command( "[O']",      lits.getLit( "O_CAP_ACUTE" ),   TokTyp.O_CAP_ACUTE       ));
        load( new Command( "[O^]",      lits.getLit( "O_CAP_CIRCUMFLEX" ),TokTyp.O_CAP_CIRCUMFLEX));
        load( new Command( "[O`]",      lits.getLit( "O_CAP_GRAVE" ),   TokTyp.O_CAP_GRAVE       ));
        load( new Command( "[O\"]",     lits.getLit( "O_CAP_UMLAUT" ),  TokTyp.O_CAP_UMLAUT      ));
        load( new Command( "[registered]",lits.getLit( "REGISTERED_TRADEMARK" ),TokTyp.REGISTERED));
        load( new Command( "[ssharp]",  lits.getLit( "S_SHARP" ),       TokTyp.S_SHARP           ));
        load( new Command( "[u']",      lits.getLit( "U_ACUTE" ),       TokTyp.U_ACUTE           ));
        load( new Command( "[u^]",      lits.getLit( "U_CIRCUMFLEX" ),  TokTyp.U_CIRCUMFLEX      ));
        load( new Command( "[u`]",      lits.getLit( "U_GRAVE" ),       TokTyp.U_GRAVE           ));
        load( new Command( "[u\"]",     lits.getLit( "U_UMLAUT" ),      TokTyp.U_UMLAUT          ));
        load( new Command( "[U']",      lits.getLit( "U_CAP_ACUTE" ),   TokTyp.U_CAP_ACUTE       ));
        load( new Command( "[U^]",      lits.getLit( "U_CAP_CIRCUMFLEX" ),TokTyp.U_CAP_CIRCUMFLEX));
        load( new Command( "[U`]",      lits.getLit( "U_CAP_GRAVE" ),   TokTyp.U_CAP_GRAVE       ));
        load( new Command( "[U\"]",     lits.getLit( "U_CAP_UMLAUT" ),  TokTyp.U_CAP_UMLAUT      ));
    }

    /**
     * Returns the type of the command based on looking up the command lexeme
     * @param key  command lexeme to look up
     * @return the command if found; otherwise null
     */
    public Command get( final String key )
    {
        if ( key == null  ) {
            return( null );
        }

        Command value = null;
        try
        {
            value = commands.get( key );
        }
        catch( ClassCastException e)
        {
            final Literals lits = Literals.getInstance();
            GDD.getInstance().getLogger().severe(
                    lits.getLit( "LOOKUP_COMMAND_ERROR" ) + " " +
                    lits.getLit( "ABORTING" ));
            System.exit( Status.FATAL_ERR );
        }
        return( value );
    }

    //===== Methods for testing only (FTO) =======/

    /**
     * For testing only (FTO). Returns the commands that have been entered
     * @return  commands in a TreeMap
     */
    public TreeMap<String, Command> FTOgetCommands()
    {
        return( commands );
    }
}

