/*--
 * (c) Copyright 2007 Pacific Data Works LLC. All Rights Reserved.
 * Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 **/

package platypus;

/**
 * Class for processing commands with numeric parameters. It accepts the number/value
 * substring, and validates the number and the units; it then converts the units to points.
 * @author alb
 */
public class CommandValue
{
    /** amount in units */
    private float amount;

    /** the type of units (lines, pts, inches, centimeters) */
    private PlatyConst.unit units;

    /** the line number in the input text where this was value was set/used */
    private int lineNumber;

    /**
     * Creates a generic CommandValue and sets it to default values
     */
    public CommandValue()
    {
        this.amount = 0;
        this.units = PlatyConst.unit.ERROR;
        this.lineNumber = GID.getInstance().getCurrLine().getLineNumber();
    }

    /**
     * Creates a CommandValue and sets it to inAmount. Units default to points.
     * @param inAmount the value of the units.
     */
    public CommandValue( float inAmount )
    {
        this();
        this.amount = inAmount;
        this.units = PlatyConst.unit.POINT;
    }

    /**
     * Constructor that is passed a value+unit substring and creates a CommandValue
     * object containing fields for value and units.
     * @param s the unit and value substring that needs to be parsed
     * @param commandName the command containing s. Used for error message only
     */
    public CommandValue( final String s, final String commandName )
    {
        this();

        ParsedUnit pu = Parser.getUnitValue( s );

        // on error, we give an error message and set amount to Float.MIN_VALUE
        if ( pu.getUnit() == PlatyConst.unit.ERROR ) {
            stateError( commandName );
            this.amount = Float.MIN_VALUE;
            return;
        }
        else {
            this.units = pu.getUnit();
        }

       // now convert all units to points
       // PlatyConst pc = new PlatyConst();

        if ( this.units == PlatyConst.unit.NONE ) {
            this.amount = pu.getAmount();
        }
        else {
            this.amount = PlatyConst.convertUnitsToPoints( pu.getUnit(), pu.getAmount() );
            this.units  = PlatyConst.unit.POINT;
        }
    }

    /**
     * Prints some kind of error message. Should be overwritten by implementing class.
     * @param s the command string originally passed to the constructor
     * @return a string containing the error statement, or null in case of error.
     */
    public String stateError( String s )
    {
        return( null );
    }

    /**
     * Dumps content of data items in user-friendly format. Should be overwritten
     * @return  a blank. Should be overwritten by subclass.
     */
    public String dump()
    {
        return( " " );
    }

    /**
     * return the number of units
     * @return units
     */
    public PlatyConst.unit getUnits()
    {
        return( units );
    }

    /**
     * return the amount of units
     * @return amount
     */
    public float getAmount()
    {
        return( amount );
    }

    /**
     * return the line number where the command was found
     * @return lineNumber
     */
    public int getLineNumber()
    {
        return( lineNumber );
    }

    public void setAmountInPoints( final float points )
    {
        amount = points;
        units = PlatyConst.unit.POINT;
        lineNumber = GID.getInstance().getCurrLine().getLineNumber();
    }
}
