/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-08 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus;

import java.util.logging.ConsoleHandler;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * GDD is the Global Document Data class. It contains numerous properties
 * relating to the output document state, as well as to Platypus' state.
 * Implemented using the Singleton pattern, as it is a one-time global item.
 *
 * @see GID for a similar class for the input document (GID = Global Input Data)
 * @author alb
 */
public class GDD
{
    /** instance of singleton that fetches the literals from the resource bundle */
    private final Literals lits = Literals.getInstance();

    /** class containing all the output routines for each output format */
    private Outfile outfile = null;

    /** type of output file (such as PDF, HTML, etc). */
    public enum FileType { IS_DOC, IS_HTML, IS_PDF }

    private CommandTable        commandTable;

    private Alignment           alignment;
    private Columns             columns;
    private CommandProcessor    commandProcessor;
    private int                 currentColumn;
    private PlatyFont           currentFont = null;
    private OutputFormatObjectFactory formatFactory;
    private Leading             leading;
    private PageFooter          footer;
    private Margins             margin;
    private Value<Boolean>      noIndent;
    private Counter             pageCounter;            // count of actual pages created
    private Counter             pageNumber;             // number that prints on the page
    private PageNumberFormat    pageNumberFormat;
    private PageSize            pageSize;
    private ParaIndent          paraIndent;
    private ParaSkip            paraSkip;
    private Strikethru          strikethru;
    private Underline           underline;

    // command-line switches 

    private boolean clVerbose  = false;                 // verbose switch
    private boolean clVVerbose = false;                 // very-verbose switch (implies -verbose)

    /** the only logger used in Platypus */
    private Logger logger  = null;

    /**
     * private constructor used for singleton pattern.
     * Initializes document properties to defaults.
     */
    private GDD()
    {
        initializeGDD();
    }

    /**
     * Initialize values. This is broken out separately due to possibility that
     * we might want to chain jobs, so then re-initializing GDD will be necessary.
     * Note:
     *    columns is initialized in OutfilePdf. It might eventually migrate there.
     *    logger  is initialized in Platypus.
     */
    public void initializeGDD()
    {
        commandTable      = new CommandTable();

        leading           = new Leading();    // must precede ParaSkip, which depends on leading.

        alignment         = new Alignment();
        currentColumn     = 0;
        formatFactory     = new OutputFormatObjectFactory();
        margin            = new Margins();
        noIndent          = new Value<Boolean>( false );
        pageCounter       = new Counter();
        pageNumber        = new Counter();
        pageNumberFormat  = new PageNumberFormat();
        pageSize          = new PageSize();
        paraIndent        = new ParaIndent( PlatyConst.DEFAULT_PARA_INDENT_PTS ); // no indent
        paraSkip          = new ParaSkip( leading.getAmount() );   // default is to skip 1 line.
        strikethru        = new Strikethru();
        underline         = new Underline();

        // new PlatyFont() is expensive due to registering font directories, so s/be done ony 1x.
        if ( currentFont == null ) {
            currentFont = new PlatyFont();
        }
        else {
            currentFont.setToDefault();
        }

        clVerbose         = false;          // command-line switch
        clVVerbose        = false;          //    "      "    "
    }

    /**
     * set up all the classes that depend on the output file format
     * @param fileType type of output file
     */
    public void setFileTypeDependentClasses( FileType fileType )
    {
        setOutfile( formatFactory.CreateOutfile( fileType ));
        setCommandProcessor( formatFactory.CreateCommandProcessor( fileType ));
        setFooter( formatFactory.CreateFooter( fileType ));
    }

    private static final GDD thisInstance = new GDD(); // we instantiate eagerly, not lazily

    /**
     * The function used by singleton pattern to return handle to this class
     * @return handle to this instance of GDD class
     */
    public static GDD getInstance()
    {
        return thisInstance;
    }

    /**
     * Dumps the GDD to stdout. Implements the [dump:all] command.
     * TODO: add levels of dumping, so that different levels of detail can be shown
     */
    public void dumpGDD()
    {
        System.out.println( "\n========== (@" + lits.getLit( "LINE" ) + " " +
                            GID.getInstance().getCurrLine().getLineNumber() + ") ==============\n");
        System.out.println( pageSize.dump() );
        
        System.out.println( margin.dump( Margins.TOP ));
        System.out.println( margin.dump( Margins.BOTTOM ));
        System.out.println( margin.dump( Margins.LEFT ));
        System.out.println( margin.dump( Margins.RIGHT ));
        System.out.println( leading.dump() );
        System.out.println( alignment.dump() );
        System.out.println( paraIndent.dump() );
        System.out.println( paraSkip.dump() );
        System.out.println( strikethru.dump() );
        System.out.println( underline.dump() );
        System.out.println( "page count " + getPageCounter().dump() );
        System.out.println( Platypus.macros.toString() );
    }

    /**
     * set up the logger we'll use and give it our own formatting.
     * This logger writes to the console only.
     */
    public void setupLogger()
    {
        logger = Logger.getLogger( "org.pz.platypus.Platypus" );

        ConsoleHandler consh = new ConsoleHandler();
        consh.setLevel( Level.ALL );
        consh.setFormatter( new LogFormatter() );
        logger.addHandler( consh );
        logger.setUseParentHandlers( false );

        // By default we log only errors and warnings. To get info-level messages, the
        // user must specify -verbose on command line. To get all info (mostly for debug),
        // user must specify -vverborse on command line. (-vverbose = very verbose)
        logger.setLevel( Level.WARNING );
    }

    //=== getters and setters ===

    public Alignment getAlignment() {
        return alignment;
    }

    public void setAlignment( final Alignment alignment ) {
        this.alignment = alignment;
    }

    public Columns getColumns() {
        return columns;
    }

    public void setColumns( final Columns columns ) {
        this.columns = columns;
    }

    public CommandProcessor getCommandProcessor() {
        return commandProcessor;
    }

    public void setCommandProcessor( final CommandProcessor commandPr ) {
        this.commandProcessor = commandPr;
    }

    public CommandTable getCommandTable() {
        return commandTable;
    }

    public void setCommandTable( final CommandTable commandTable ) {
        this.commandTable = commandTable;
    }

    public int getCurrentColumn() {
        return currentColumn;
    }

    public void setCurrentColumn( final int currentColumn ) {
        this.currentColumn = currentColumn;
    }

    public PlatyFont getCurrFont() {
        return currentFont;
    }

    public void setCurrFont( PlatyFont newFont )
    {
        if ( newFont != null ) {
            currentFont = newFont;
        }
    }

    public OutputFormatObjectFactory getFormatFactory() {
        return formatFactory;
    }

    public void setFormatFactory( final OutputFormatObjectFactory formatFactory ) {
        this.formatFactory = formatFactory;
    }

    public PageFooter getFooter() {
        return footer;
    }

    public void setFooter( final PageFooter footer ) {
        this.footer = footer;
    }

    public Leading getLeading() {
        return leading;
    }

    public void setLeading( final Leading leading ) {
        this.leading = leading;
    }

    public Margins getMargin() {
        return margin;
    }

    public void setMargin( final Margins margin ) {
        this.margin = margin;
    }

    public Value<Boolean> getNoIndent() {
        return noIndent;
    }

    public void setNoIndent( final Value<Boolean> noIndent ) {
        this.noIndent = noIndent;
    }

    public Counter getPageCounter() {
        return pageCounter;
    }

    public Counter getPageNumber() {
        return pageNumber;
    }

    public PageNumberFormat getPageNumberFormat() {
        return pageNumberFormat;
    }

    public PageSize getPageSize() {
        return pageSize;
    }

    public void setPageSize( final PageSize pageSize ) {
        this.pageSize = pageSize;
    }

    public ParaIndent getParaIndent() {
        return paraIndent;
    }

    public void setParaIndent( final ParaIndent paraIndent ) {
        this.paraIndent = paraIndent;
    }

    public ParaSkip getParaSkip() {
        return paraSkip;
    }

    public void setParaSkip( final ParaSkip paraSkip ) {
        this.paraSkip = paraSkip;
    }

    public Strikethru getStrikethru() {
        return strikethru;
    }

    public void setStrikethru( final Strikethru strikethru ) {
        this.strikethru = strikethru;
    }

    public Underline getUnderline() {
        return underline;
    }

    public void setUnderline( final Underline underline ) {
        this.underline = underline;
    }

    public boolean isClVerbose() {
        return clVerbose;
    }

    public void setClVerbose( final boolean clVerbose ) {
        this.clVerbose = clVerbose;
    }

    public boolean isClVVerbose() {
        return clVVerbose;
    }

    public void setClVVerbose( final boolean clVVerbose ) {
        this.clVVerbose = clVVerbose;
    }

    public Logger getLogger() {
        return logger;
    }

    public void setLogger( final Logger logger ) {
        this.logger = logger;
    }
        
    public Outfile getOutfile() {
        return outfile;
    }

    public void setOutfile( final Outfile outfile ) {
        this.outfile = outfile;
    }

}