/* (c) Copyright 2006 Pacific Data Works LLC. All Rights Reserved.
 * Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 *
 * For more information about Platypus, go to http://platypus.pz.org
 * TODO: add counter identifying where we are in the input file (at which token)
 */
package platypus;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

/**
 * Handles input file I/O functions.
 *
 * Note that in Platypus, all input files are read into memory (into an array of lines)
 *          when opened. They are then closed. And we work from the array of lines.
 *          
 * @author alb          
 */
public class Infile
{
	/** the name of the input file */
    private String filename;

    /**  array of ReadLines into which Infile is parsed */
    ArrayList<ReadLine> textLines;  
    
    /** the line number in the original input file (from ReadLine) */
    int currLineNumber;             

    /** used only for unit testing purposes */
    int commentLines = 0; 
    
    GID gid;
    GDD gdd;

    /**
     * this is the index to the textLines ArrayList that holds the lines of text. It tells us where
     * we are in the ArrayList for purposes of reading the next line.
     */
    private static int textLineIndex;

    public Infile() {};

    public Infile( final String name )
    {
        filename = name;
        textLines = new ArrayList<ReadLine>();
        gid = GID.getInstance();
        gdd = GDD.getInstance();
    }

    /**
     * Reads the input file on a line basis, adding each line to an ArrayList for
     * later processing. Lines where the first character is a comment are skipped
     * (that is, not saved) in this process.
     *
     * @return the number the total number of lines read or an error code on exception
     */
    public int readFile()
    {
        int lineCount = 0;
        BufferedReader in;
        int retVal = 0;

        final File thisFile = new File( filename );
        try
        {
            FileReader fr = new FileReader( thisFile );
            in = new BufferedReader( fr );
        }
        catch( FileNotFoundException e)
        {
            return( Status.FILE_NOT_FOUND_ERR ); // all files are known to exist by this point.
        }

        boolean atEOF = false;
        while( ! atEOF )
        {
            ReadLine rl = new ReadLine();
            try
            {
                rl.setContent( in.readLine() );
                if ( rl.getLineContent() == null ) {
                    atEOF = true;
                }
                else {
                    rl.setLineNumber( ++lineCount );

                    // remove comment lines (those that begin with %%, where the %% is not
                    // the closing of a comment block, such as part of: %%%]
                    if ( rl.getLineContent().startsWith( "%%" ) &&
                            ! startsWithEndOfBlockComment( rl.getLineContent() )) {
                    	commentLines++;      // counted only for unit-testing purposes
                    }
                    else {
                    	textLines.add( rl );        //TODO put add in try/catch block
                    }
                }           
            }
            catch( IOException e )
            {
                retVal = Status.IO_ERR;
            }
        }
        
        // now close the input filestream
        try
        {
        	in.close();
        }
        catch( IOException e )
        {
        	retVal = Status.IO_ERR;
        }
        //-- debug --  TODO: consider using this if we implement a -debug command-line switch
        //
        // gdd.logger.finest( "at end of ReadFile. Have " + lineCount + "Lines" );
        //
        // for( int i = 0; i < textLines.size(); i++)
        //     gdd.logger.fine( textLines.get(i).lineNumber + ": " + textLines.get(i).lineContent );
        //-- end debug --
        
        // return either the linecount or the error code
        return(( retVal == Status.IO_ERR ) ? retVal : lineCount );
    }

    /**
     * Determines whether a line that starts with %% is part of a closing comment block.
     * Comment blocks close with %%], where the number of %% is some number >= 2
     * @param line the line to check
     * @return true if pattern found at beginning of line, false otherwise
     */
    boolean startsWithEndOfBlockComment( final String line )
    {
        if ( line == null ) {
            return( false );
        }
        
        for( int i = 2; i < line.length(); i++ )
        {
            char c = line.charAt( i );
            if ( c == '%') {
                continue;
            }
            else 
            if ( c == ']' ) {
                return( true );
            }
            else {
                return( false );
            }
        }

        return( false );
    }

    /**
     * Remove block comments from textLines array. Block comments open with [%%
     * and close with %%] The number of %'s can be up to 100 and opening and
     * closing asterisk counts must match. So [%%% not right %%] is invalid, but
     * [%%% is right %%] %%%] is a valid comment. The interior %%] is ignored.
     * Block comments close upon finding a closing match regardless of where in the
     * file it occurs.
     *
     * @return number of block comments found. UNCLOSED_COMMENT_BLK if an unclosed comment exists
     */
    public int stripBlockComments()
    {
        String textline;
        int j = 0;
        int k = 0;
        int blockCommentsFound = 0;
        boolean inBlockComment = false;
        String endOfCommentMark = "";

        for ( int i = 0; i < textLines.size(); i++ )
        {            
            try
            {
                textline = textLines.get( i ).getLineContent();
            }
            catch( IndexOutOfBoundsException e )
            {
                if ( inBlockComment ) {
                    gdd.getLogger().severe( Literals.getInstance().getLit(
                            "ERR_COMMENT_BLOCK_NOT_CLOSED" ));
                    return( Status.UNCLOSED_COMMENT_BLK );
                }
                else {
                    return( blockCommentsFound );
                }
            }

            if ( inBlockComment ) {
                k = textline.indexOf( endOfCommentMark );
                if ( k == -1 ) { // EOcomment not found, so delete this entire line (it's a comment)
                    textLines.remove( i );
                    --i;    // otherwise we skip a line due to the removal
                    continue;
                }
                else {
                    // EOcomment found:
                    // check to see that the mark is not part of a larger closing mark (that is,
                    // if we find %%%] make sure it's not really part of %%%%]).
                    if ( k == 0 || ( textline.charAt( k-1 ) != '%' )) {
                        // it's the correct mark, so delete the part upto and including the mark
                        // and replace the current line with what remains of the text.
                        // then decrease i, so that the next pass through this main loop
                        // picks up this new line fragment.
                    	
                        final ReadLine rlnew = new ReadLine();
                        rlnew.setLineNumber( textLines.get( i ).getLineNumber() );
                        rlnew.setContent( textline.substring( k + endOfCommentMark.length() ));
                        textLines.remove( i );
                        textLines.add( i, rlnew );
                        i--;
                        inBlockComment = false;
                        continue;
                    }
                    else {
                    	// the mark was part of a larger closing mark. To continue checking the rest
                    	// of the line, we break the line: everything up and including to the mark we
                    	// just matched is deleted. We then replace the current line with what remains
                    	// of the original line. The next pass through the main loop will pick up
                    	// and search through the second line fragment
                    	
                        final ReadLine rlnew = new ReadLine();
                        rlnew.setLineNumber( textLines.get( i ).getLineNumber() );
                        rlnew.setContent( textline.substring( k + endOfCommentMark.length() - 1 ));
                        textLines.set( i, rlnew );
                        i--; // so that next pass picks up this remaining piece.
                        continue;
                    }
                }               
            }
            else {
                j = textline.indexOf( "[%%" );
                if ( j == -1 ) {	// not found
                    continue;
                }
                else
                if ( j > 0 && textline.charAt(j-1) == '\\' ) {
                        continue;   // if [ preceded by \, not a comment (for the nonce)
                }                 //TODO: if preceded by multiple \, could still be a comment block
                else
                {   
                	// we have a legitimate start to a block comment.
                    inBlockComment = true;
                    blockCommentsFound++;

                    // compute what the end of block must look like, so we can search for it.
                    endOfCommentMark = composeEndOfComment( j, textline );

                    // break the current line at the match. Place everything after the match
                    // in a new line inserted right after the current line, so that the main loop
                    // searches it next.
                    
                    final ReadLine rlnew = new ReadLine();
                    rlnew.setLineNumber( textLines.get( i ).getLineNumber() );
                    rlnew.setContent( textline.substring( j + endOfCommentMark.length() ));
                    textLines.add( i + 1, rlnew );

                    // if the current textLine started with a comment block, delete the textLine
                    // (as the part without the comment starter has already been added to TextLines),
                    // otherwise, replace current textLine with what preceded the comment marker.
                    
                    if ( j== 0 ) {
                        textLines.remove( i );
                        i--; // if we don't decrement i the newly added line will be skipped
                             // because the remove will put the new line in the current slot.
                    }
                    else {
                        textLines.get( i ).setContent( textline.substring( 0, j ));
                    }
                    continue;
                }
            }
        }
        return( blockCommentsFound );
    }

    /**
     * Accepts a line of text and an index of the first character of an opening comment block.
     * It then creates a string that contains the corresponding closing comment marker.
     *
     * @param start    index into textline that points to where the opening marker begins
     * @param textline the line of text with the opening block comment marker embedded
     * @return   a string containing the corresponding endOfComment block marker
     */
    String composeEndOfComment( final int start, final String textline )
    {
        int i = start + 1; // skip the opening '[' character
        int percents = 0;  // the number of % characters in the comment opening mark

        for( ;  i < textline.length(); i++ )
        {
            if ( textline.charAt( i ) == '%' ) {
                percents++;
            }
            else {
                break;
            }
        }
        
        final StringBuffer endOfComment = new StringBuffer();
        for( i = 0; i < percents; i++ )
            endOfComment.append( '%' );
        endOfComment.append( ']' );
        return( endOfComment.toString() );
     }

    /**
     * Gets the next line in the ArrayList of parsed lines from the input file. It skips over
     * comment lines. Empty lines are replaced by an end-of-paragraph token. The line is then
     * trimmed of leading and trailing whitespace and placed in gid.currLine.
     *
     * @return Status.AT_EOF at EOF, or Status.NO_ERROR if no problem occurred.
     */
    public int getLine()
    {
        ReadLine rl;

        while( true )
        {
            try
            {
                rl = textLines.get( textLineIndex );
            }
            catch( IndexOutOfBoundsException e )
            {
                return( Status.AT_EOF );
            }

            gid.getCurrLine().setContent( rl.getLineContent().trim() );
            gid.getCurrLine().setLineNumber( rl.getLineNumber() );

            // if line is empty, treat as a CR/LF = new paragraph = []
            if ( gid.getCurrLine().getLineContent().length() == 0 ) {
                // replace the input line with the code for new paragraph
                gid.getCurrLine().setContent( "[]" );
            }

            // if it's a comment, read another line.
            if ( gid.getCurrLine().getLineContent().startsWith( "%%" )) {
                textLineIndex++;
                continue;
            }
            else {
                break;
            }
        }
        return( Status.NO_ERROR );
    }

    /**
     * Verifies the existence of a specific file.
     * @return  boolean indicating whether file exists
     */
    public boolean doesExist()
    {
        final File thisFile = new File( filename );
        return( thisFile.exists() );
    }

    /**
     * Verifies that the file can be read.
     * @return  boolean indicating whether file exists
     */
    public boolean isReadable()
    {
        final File thisFile = new File( filename );
        return ( thisFile.canRead() );
    }

    /**
     * convenience function for incrementing the pointer to the next line in the input stream
     */
    public void incrementLineCount()
    {
        textLineIndex++;
    }

    public String getFilename()
    {
        return filename;
    }

    public ArrayList<ReadLine> getTextLines()
    {
        return( textLines );
    }

    /**
     * returns the current line number being handled by Infile
     * @return the current line number
     */
    public int getLineNumber()
    {
        return( currLineNumber );
    }

    //======= testing-only functions =========//

    /**
     * used primarily (exclusively?) in unit tests
     * @return the number of the current line
     */
    public int getLineCount()
    {
        return textLineIndex;
    }
}
