/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-08 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus;

/**
 * Handles the leading for text, that is, the path between baselines of text.
 * It's a paragraph-level operation.
 *
 * @author alb
 */
public class Leading
{
    private float   amount = 0f;
    private int     line = 0;
    private boolean leadingChanged = false;

    public Leading()
    {
        amount = PlatyConst.DEFAULT_LEADING_PTS;
        line = 0;
        leadingChanged = false;
    }

    public void setAmount( final float newLeading )
    {
        if ( newLeading != amount ) {
            amount = newLeading;
            leadingChanged = true;
            line = GID.getInstance().getCurrLine().getLineNumber();
        }
    }

    /**
     * Uses the parsed command's quantity and unit-type to determine number of points,
     * then calls setAmount( points ) above to do the actual value setting.
     * @param puLeading the parsed unit containing amount and unit-type
     */
    public void setAmount( final ParsedUnit puLeading )
    {
        if ( puLeading == null ) {
            return;
        }

        if ( puLeading.getUnit() != PlatyConst.unit.ERROR ) {
            float points =
                    PlatyConst.convertUnitsToPoints( puLeading.getUnit(), puLeading.getAmount() );
            if ( points != Float.NaN ) {
                setAmount( points );
            }
        }
    }

    public float getAmount()
    {
        return( amount );
    }

    public void setLeadingChangedSwitch( final boolean yesNo )
    {
        leadingChanged= yesNo;
    }
    
    public boolean isLeadingChanged()
    {
        return leadingChanged;
    }

    public int getLineNumber()
    {
        return( line );
    }

    /**
     * Creates a string containing the line # and the value of the bottom margin when last set
     * @return formatted string suitable for use in dump()
     */
    public String dump()
    {
        final Literals lits = Literals.getInstance();

        final StringBuffer sb = new StringBuffer();
        sb.append( lits.getLit( "LEADING" ))
          .append( " (" )
          .append( lits.getLit( "LINE" ))
          .append( " " )
          .append( getLineNumber() )
          .append( ") " )
          .append( DumpValue.pointsFormatted( getAmount() ))
          .append(  " " )
          .append( lits.getLit( "POINTS" ))
          .append( " = "  )
          .append( DumpValue.pointsToInches( getAmount() ));
        return( sb.toString() );
    }

    public String errorMessage( final String token )
    {
        final GDD gdd = Platypus.gdd;
        final GID gid = Platypus.gid;
        final Literals lits = Platypus.lits;

        final StringBuffer sb = new StringBuffer();
        sb.append( lits.getLit( "ERR_INVALID_LEADING" ))
          .append( " " )
          .append( token )
          .append( " " )
          .append( lits.getLit( "AT_LINE" ))
          .append( ": " )
          .append( gid.getCurrLine().getLineNumber() )
          .append(  " " )
          .append( lits.getLit( "IGNORED" ));

        gdd.getLogger().warning( sb.toString() );

        return( sb.toString() );
    }
}
