/*--
 *  Command value class for handling page margins
 *
 *  (c) Copyright 2006-7 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */
package platypus;
/**
 * Uber-class for page margins.
 * @author alb
 */
public class Margins
{
    private Value<Float>[] allMargins;
    private Value<Boolean> marginsMirrored;

    public static final int TOP     = 0;
    public static final int BOTTOM  = 1;
    public static final int LEFT    = 2;
    public static final int RIGHT   = 3;
    public static final int INNER   = 4;    // not currently supported
    public static final int OUTER   = 5;    // not currently supported

    // convenience constants identifying the range of margins currently supported
    private static final int LEAST_MARGIN = 0;
    private static final int MOST_MARGIN  = 3;

    public Margins()
    {
        allMargins = new Value[4];

        allMargins[TOP]     = new Value<Float>( PlatyConst.DEFAULT_MARGIN_PTS );
        allMargins[BOTTOM]  = new Value<Float>( PlatyConst.DEFAULT_MARGIN_PTS );
        allMargins[LEFT]    = new Value<Float>( PlatyConst.DEFAULT_MARGIN_PTS );
        allMargins[RIGHT]   = new Value<Float>( PlatyConst.DEFAULT_MARGIN_PTS );

        marginsMirrored     = new Value<Boolean>( false );
    }

    public int setSize( final int which, final float size )
    {
        if( which < LEAST_MARGIN || which > MOST_MARGIN ) {
            return( Status.ERR_MARGIN_SPECIFIED );
        }

        if ( size < 0 || size > GDD.getInstance().getPageSize().getWidth() - 1.0f ) {
            return( Status.ERR_MARGIN_SIZE );
        }

        allMargins[which].setValue( size );
        return( Status.OK );
    }

    public float getSize( final int which )
    {
        if( which < LEAST_MARGIN || which > MOST_MARGIN ) {
            return( Status.ERR_MARGIN_SPECIFIED );
        }

        return( Float.valueOf( allMargins[which].getValue() ));
    }

    public int getLineNumber( final int which )
    {
        if( which < LEAST_MARGIN || which > MOST_MARGIN ) {
            return( Status.ERR_MARGIN_SPECIFIED );
        }

        return( allMargins[which].getLineNumber() );
    }

    public void setMarginsMirrored( final boolean onOff ) {
        marginsMirrored.setValue( onOff );
    }

    public boolean areMarginsMirrored() {
        return Boolean.valueOf( marginsMirrored.getValue() );
    }

    public String dump( final int which )
    {
        if( which < LEAST_MARGIN || which > MOST_MARGIN ) {
            return( null );
        }

        Literals lits = Literals.getInstance();
        GDD gdd = GDD.getInstance();
        StringBuffer sb = new StringBuffer();

        switch( which )
        {
            case TOP:    sb.append( lits.getLit( "TOP_MARGIN"    )); break;
            case BOTTOM: sb.append( lits.getLit( "BOTTOM_MARGIN" )); break;
            case LEFT:   sb.append( lits.getLit( "LEFT_MARGIN"   )); break;
            case RIGHT:  sb.append( lits.getLit( "RIGHT_MARGIN"  )); break;
        }

        sb.append( " ")
          .append( allMargins[which].dump() )
          .append( " ")
          .append( lits.getLit( "POINTS" ))
          .append( " = " )
          .append( DumpValue.pointsToInches( gdd.getMargin().getSize( which )));

        return( sb.toString() );
    }
}
