/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-08 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus;

/**
 * Contains the format for a page number.
 *
 * @author alb
 */
public class PageNumberFormat
{
    /** specifies the possible formats for the page number */
    public enum format { ROMAN_LOWER, ROMAN_CAPITAL, ARABIC };

    private format  currFormat;
    private int     line = 0;

    public PageNumberFormat()
    {
        currFormat = format.ARABIC;
        line = 0;
    }

    /**
     * Formats the page number using the currently select format
     *
     * @param pageNumber the page number to format
     * @return the formatted page number as a string, or null if an error occurs
     */
    public String formatPageNumber( final int pageNumber )
    {
        return( formatPageNumber( pageNumber, currFormat ));
    }

    /**
     * Format the page number into a string according to the selected format
     * @param pageNumber the page number to format
     * @param format the format to apply
     * @return the formatted page number as a string, or null if an error occurs
     */
    public String formatPageNumber( final int pageNumber, final PageNumberFormat.format format )
    {
        if ( pageNumber < 1 || format == null ) {
            return( null );
        }

        switch( format )
        {
            case ARABIC:
                return( formatArabic( pageNumber ));
            case ROMAN_CAPITAL:
                return( formatRomanCapital( pageNumber ));
            case ROMAN_LOWER:
                return( formatRomanLower( pageNumber ));
            default:
                // in case of error, don't balk, just make it arabic.
                return( formatArabic( pageNumber ));
        }
    }

    /**
     * Format a page number using arabic numbers (regular digits)
     * @param number number to format
     * @return formatted number as a string
     */
    public String formatArabic( final int number )
    {
        return( Integer.toString( number ));
    }

    /**
     * Convert an integer > 0 to upper case roman numerals
     * @param integer number to convert
     * @return the number as a roman numeral in caps, or null if an error occurs
     */
    public String formatRomanCapital( final int integer )
    {
        if ( integer < 1 ) {
            return( null );
        }

        StringBuffer sb = new StringBuffer( 9 );

        final int[]    values  = { 1000,  900,  500,  400,  100,   90,
                                   50,    40,   10,    9,    5,    4,    1 };

        final String[] letters = { "M",  "CM",  "D",  "CD", "C",  "XC",
                                   "L",  "XL",  "X",  "IX", "V",  "IV", "I" };

        int number = integer;

        for( int i = 0; i < values.length; i++ )
        {
            while( number >= values[i] )
            {
                sb.append( letters[i] );
                number -= values[i];
            }
        }

        return sb.toString();
    }

    /**
     * Accepts an integer and converts it to lower-case roman numerals. First,
     * it uses formatRomanCapital() to convert number to capital roman numerals,
     * then converts the digits to lower case. Does not use toLower() because of
     * possible internationalization issues. This direct mapping guarantees a
     * correct result.
     * @param number to convert
     * @return formatted string, or null if an invalid character is encountered.
     */
    public String formatRomanLower( final int number )
    {
        StringBuffer sb = new StringBuffer( 9 );

        final String RomanCaps = formatRomanCapital( number );

        if ( RomanCaps == null ) {
            return( null );
        }

        for ( int i = 0; i < RomanCaps.length(); i++ )
        {
            char c = RomanCaps.charAt( i );
            switch( c )
            {
                case 'C': sb.append( 'c' ); break;
                case 'D': sb.append( 'd' ); break;
                case 'I': sb.append( 'i' ); break;
                case 'L': sb.append( 'l' ); break;
                case 'M': sb.append( 'm' ); break;
                case 'V': sb.append( 'v' ); break;
                case 'X': sb.append( 'x' ); break;
                default: return( null );
            }
        }
        return( sb.toString() );
    }

    public void setFormat( final format newFormat )
    {
        if ( newFormat != currFormat ) {
            currFormat = newFormat;
            line = GID.getInstance().getCurrLine().getLineNumber();
        }
    }

    public format getFormat()
    {
        return( currFormat );
    }

    public int getLineNumber()
    {
        return( line );
    }

    /**
     * Creates a string containing the line # and the value of the bottom margin when last set
     * @return formatted string suitable for use in dump()
     */
    public String dump()
    {
        final Literals lits = Literals.getInstance();

        final StringBuffer sb = new StringBuffer();
        sb.append( lits.getLit( "PAGE_NUMBER_FORMAT" ))
          .append( " (" )
          .append( lits.getLit( "LINE" ))
          .append( " " )
          .append( getLineNumber() )
          .append( ") " );

        switch( currFormat ) {
            case ARABIC:        sb.append( lits.getLit( "ARABIC" ));            break;
            case ROMAN_CAPITAL: sb.append( lits.getLit( "ROMAN_CAPITAL" ));     break;
            case ROMAN_LOWER:   sb.append( lits.getLit( "ROMAN_LOWER_CASE" ));  break;
            default:            sb.append( lits.getLit( "INVALID" ));           break;
        }

        return( sb.toString() );
    }

    public String errorMessage( final String token )
    {
        final GDD gdd = Platypus.gdd;
        final GID gid = Platypus.gid;
        final Literals lits = Platypus.lits;

        final StringBuffer sb = new StringBuffer();
        sb.append( lits.getLit( "ERR_INVALID_PAGE_NUMBER_FORMAT" ))
          .append( " " )
          .append( token )
          .append( " " )
          .append( lits.getLit( "AT_LINE" ))
          .append( ": " )
          .append( gid.getCurrLine().getLineNumber() )
          .append(  " " )
          .append( lits.getLit( "IGNORED" ));

        gdd.getLogger().warning( sb.toString() );

        return( sb.toString() );
    }
}