/*--
 * (c) Copyright 2006-07 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 *
 *  For more information about platypus, go to http://platypus.pz.org
 * //TODO: this class is intimately tied to iText. Decouple these. -alb
 */

package platypus;

import java.util.HashMap;

import com.lowagie.text.Rectangle;

/**
 * All functions relating to setting, getting, and dumping the page size
 * @author alb
 */
public class PageSize
{
    /** page width in points */
    private float width;

    /** page height in points */
    private float height;

    /** line where page size is specified */
    private int lineNumber;

    /** HashMap that maps strings of page-size names to iText rectangles */
    private HashMap<String, Rectangle> pageNames;

    /**
     * Default constructor simply uses the default values (letter, or 8.5" x 11")
     */
    public PageSize()
    {
        width = PlatyConst.DEFAULT_PAGE_WD_PTS;
        height = PlatyConst.DEFAULT_PAGE_HT_PTS;
        lineNumber = GID.getInstance().getCurrLine().getLineNumber();
    }

    /**
     * Initialize the page size to one of the pre-written shorthand sizes (LTR, LGL, A4, etc.)
     * @param sizeName the pre-existing shorthand name for a specific page size
     * @throws IllegalArgumentException if invalid string is passed in.
     */
    public void initialize( final String sizeName ) throws IllegalArgumentException
    {
        if ( sizeName == null ) {
            throw new IllegalArgumentException(
                Literals.getInstance().getLit( "ERR_INVALID_PAGESIZE" ) + " " + sizeName );
        }

        if ( GDD.getInstance().getOutfile().hasTextBeenProcessed() ) {
            tooLateError();
            return;
        }

        pageNames = new HashMap<String, Rectangle>();
        loadPageNames();

        final String name = sizeName.toUpperCase();
        final Rectangle rect = pageNames.get( name );
        if ( rect == null ) {
            Literals lits = Literals.getInstance();
            throw new IllegalArgumentException(
                lits.getLit( "ERR_INVALID_PAGESIZE" ) + " " + sizeName +
                 " (" + lits.getLit( "LINE") + " " +
                GID.getInstance().getCurrLine().getLineNumber() + ") " + lits.getLit( "IGNORED" ));
        }
        else {
            width = rect.width();
            height = rect.height();
            lineNumber = GID.getInstance().getCurrLine().getLineNumber();
        }
    }

    /**
     * Load the page names and the corresponding iText rectangles into a hash table.
     * for more info on page sizes see: http://en.wikipedia.org/wiki/Paper_size#Loose_sizes
     * for info on ID cards sizes, see: http://en.wikipedia.org/wiki/ISO_7810
     */
    private void loadPageNames()
    {
        pageNames.put( "4A0",           new Rectangle( 4768, 6741 ));           // 1682mm x 2378mm DIN476
        pageNames.put( "2A0",           new Rectangle( 3370, 4768 ));           // 1189mm x 1682mm DIN476
        pageNames.put( "A0",            com.lowagie.text.PageSize.A0 );         // 841mm x 1189mm
        pageNames.put( "A1",            com.lowagie.text.PageSize.A1 );         // 594mm x 841mm
        pageNames.put( "A2",            com.lowagie.text.PageSize.A2 );         // 420mm x 594mm
        pageNames.put( "A3",            com.lowagie.text.PageSize.A3 );         // 297mm x 420mm
        pageNames.put( "A4",            com.lowagie.text.PageSize.A4 );         // 210mm x 297mm
        pageNames.put( "A5",            com.lowagie.text.PageSize.A5 );         // 148mm x 210mm
        pageNames.put( "A6",            com.lowagie.text.PageSize.A6 );         // 105mm x 148mm
        pageNames.put( "A7",            com.lowagie.text.PageSize.A7 );         // 74mm  x 105mm
        pageNames.put( "A8",            com.lowagie.text.PageSize.A8 );         // 52mm  x 74mm
        pageNames.put( "A9",            com.lowagie.text.PageSize.A9 );         // 37mm  x 52mm
        pageNames.put( "A10",           com.lowagie.text.PageSize.A10 );        // 26mm  x 37mm
        pageNames.put( "B0",            com.lowagie.text.PageSize.B0 );         // 1000mm x 1414mm
        pageNames.put( "B1",            com.lowagie.text.PageSize.B1 );         // 707mm x 1000mm
        pageNames.put( "B2",            com.lowagie.text.PageSize.B2 );         // 500mm x 707mm
        pageNames.put( "B3",            com.lowagie.text.PageSize.B3 );         // 353mm x 500mm
        pageNames.put( "B4",            com.lowagie.text.PageSize.B4 );         // 250mm x 353mm
        pageNames.put( "B5",            com.lowagie.text.PageSize.B5 );         // 176mm x 250mm
        pageNames.put( "B6",            com.lowagie.text.PageSize.B6 );         // 125mm x 176mm
        pageNames.put( "B7",            com.lowagie.text.PageSize.B7 );         // 88mm  x 125mm
        pageNames.put( "B8",            com.lowagie.text.PageSize.B8 );         // 62mm  x 88mm
        pageNames.put( "B9",            com.lowagie.text.PageSize.B9 );         // 44mm  x 62mm
        pageNames.put( "B10",           com.lowagie.text.PageSize.A10 );        // 31mm  x 44mm
        pageNames.put( "BUSCARD-INTL",  new Rectangle( 243 , 153 ));            // Business card int'l
        pageNames.put( "BUSCARD-US",    new Rectangle( 252, 144 ));             // Business card US
        pageNames.put( "ANSI-A",        com.lowagie.text.PageSize.LETTER );     // ANSI Letter
        pageNames.put( "ANSI-B",        com.lowagie.text.PageSize._11X17 );     // ANSI 11 x 17
        pageNames.put( "ANSI-C",        new Rectangle( 1224, 1584 ));           // ANSI 17 x 22
        pageNames.put( "ANSI-D",        new Rectangle( 1584, 2448 ));           // ANSI 22 x 34
        pageNames.put( "ANSI-E",        new Rectangle( 2448, 3168 ));           // ANSI 34 x 44
        pageNames.put( "ARCH-A",        com.lowagie.text.PageSize.ARCH_A );     // architectural A
        pageNames.put( "ARCH-B",        com.lowagie.text.PageSize.ARCH_B );     // architectural B
        pageNames.put( "ARCH-C",        com.lowagie.text.PageSize.ARCH_C );     // architectural C
        pageNames.put( "ARCH-D",        com.lowagie.text.PageSize.ARCH_D );     // architectural D
        pageNames.put( "ARCH-E",        com.lowagie.text.PageSize.ARCH_C );     // architectural E        
        pageNames.put( "HALFLETTER",    com.lowagie.text.PageSize.HALFLETTER ); // 5.5" x 8.5"
        pageNames.put( "ID-1",          com.lowagie.text.PageSize.ID_1 );       // ISO 7810 ID-1
        pageNames.put( "ID-2",          com.lowagie.text.PageSize.ID_2 );       // ISO 7810 ID-2
        pageNames.put( "ID-3",          com.lowagie.text.PageSize.ID_3 );       // ISO 7810 ID-3
        pageNames.put( "INDEX3X5",      new Rectangle( 216, 360 ));             // 3" x 5"
        pageNames.put( "INDEX4X6",      new Rectangle( 288, 432 ));             // 4" x 6"
        pageNames.put( "INDEX5X8",      new Rectangle( 360, 576 ));             // 5" x 8"
        pageNames.put( "LEDGER",        com.lowagie.text.PageSize.LEDGER );     // 17"  x 11"
        pageNames.put( "LEGAL",         com.lowagie.text.PageSize.LEGAL );      // 8.5" x 14"
        pageNames.put( "LETTER",        com.lowagie.text.PageSize.LETTER );     // 8.5" x 11"
        pageNames.put( "NOTE",          com.lowagie.text.PageSize.NOTE );       // 7.5" x 10"
        pageNames.put( "11X17",         com.lowagie.text.PageSize._11X17 );     // 11"  x 17"
    }

    /**
     * Creates a string containing the line # and the values of page size when last set
     * @return formatted string suitable for use in dump
     * TODO: Consider looking thru the pageNames table and dumping the name for the page size
     *       Note: if we do this, then loadPageNames() must be called in the constructor.
     */
    public String dump()
    {
        final Literals lits = Literals.getInstance();

        final StringBuffer sb = new StringBuffer();
        sb.append( lits.getLit( "PAGE_SIZE" ))
          .append( " (" )
          .append( lits.getLit( "LINE" ))
          .append( " " )
          .append( lineNumber )
          .append(  ") " )
          .append( DumpValue.pointsFormatted( height ))
          .append( " " )
          .append( lits.getLit( "POINTS" ))
          .append( " " )
          .append( lits.getLit( "HEIGHT_ABBREV" ))
          .append( " x " )
          .append( DumpValue.pointsFormatted( width ))
          .append( " " )
          .append( lits.getLit( "POINTS" ))
          .append( " " )
          .append( lits.getLit( "WIDTH_ABBREV" ));
        return( sb.toString() );
    }

    /**
     * Logs an error when the user tries to change the size of the page after any text
     * has been emitted. Platypus does not support this.
     *
     * @return a string containing the error message (string used primarily for testing)
     */
    public String tooLateError()
    {
        final Literals lits = Literals.getInstance();

        final StringBuffer sb = new StringBuffer();
        sb.append( lits.getLit( "ERR_PAGE_SIZE_MUST_PRECEDE_ALL_TEXT" ))
          .append( " (" )
          .append( lits.getLit( "LINE" ))
          .append( " " )
          .append( GID.getInstance().getCurrLine().getLineNumber() )
          .append( ") " )
          .append( lits.getLit( "IGNORED" ));

        GDD.getInstance().getLogger().warning( sb.toString() );

        return( sb.toString() );
    }

    public float getHeight() {
        return height;
    }

    /**
     * currently, you cannot reset page height if text has already been
     * processed. This might change in the future.
     *
     * @param newHeight
     */
    public void setHeight( final float newHeight )
    {
       if ( GDD.getInstance().getOutfile().hasTextBeenProcessed() ) {
            tooLateError();
            return;
        }

        height = newHeight;
        lineNumber = GID.getInstance().getCurrLine().getLineNumber();
    }

    public int getLineNumber() {
        return lineNumber;
    }

    public void setLineNumber( final int lineNumber ) {
        this.lineNumber = lineNumber;
    }

    public float getWidth() {
        return width;
    }

    /**
     * currently you cannot reset width if text has already been
     * processed. This might change in the future.
     *
     * @param newWidth
     */
    public void setWidth( final float newWidth )
    {
       if ( GDD.getInstance().getOutfile().hasTextBeenProcessed() ) {
            tooLateError();
            return;
        }

        width = newWidth;
        lineNumber = GID.getInstance().getCurrLine().getLineNumber();
    }      
}
