/***
 *  Platypus is (c) Copyright 2006-07 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus;

/**
 * Handles all the various parsing tasks. All public classes are static.
 */
public class Parser
{
    /**
     * Parses a substring that *starts* with the definition of a
     * quantity. So if the command is [paraindent:36pt], this routine would
     * be passed 36pt] and would return a structure containing the value 36
     * the unit-type (enum) POINT, and the length of characters gobbled; 3.
     * This method correctly handles macros in the string.
     *
     * @param inputStr string containing quantity and unit abbreviation
     * @return a ParsedUnit instance containing value and type, or error type & error message
     */
    public static ParsedUnit getUnitValue( final String inputStr )
    {
        /** the ParsedUnit we'll be filling in and returning */
        ParsedUnit ret = new ParsedUnit();

        String macroName;
        String actualValue;
        int charsScanned = 0;

        // set ret to error value. We'll override these if we get good results.
        ret.setUnit( PlatyConst.unit.ERROR );
        ret.setErrorCode( Status.INVALID_PARAM );
        ret.setCharsParsed( 0 );

        if ( inputStr == null ) {
            ret.setErrorCode( Status.INVALID_PARAM_NULL );
            return( ret );
        }

        if ( inputStr.length() == 0 ) {
            return( ret );
        }

        // check for a Platypus system macro or a user macro
        if ( inputStr.charAt( 0 ) == '$' || inputStr.charAt( 0 ) == '#' ) {
            macroName = Platypus.macros.extractMacroName( inputStr );
            charsScanned += macroName.length();
            actualValue = Platypus.macros.get( macroName );
            if ( actualValue == null ) {
                ret.setCharsParsed( charsScanned );
                return( ret );
            }
            actualValue =  actualValue.concat( inputStr.substring( macroName.length() ));
        }
        else {
            actualValue = inputStr;
        }
        
        // at this point, we have a string that starts with the true  value,
        // with all of its macros resolved, followed by the units (which might still contain macros).

        StringBuffer numericPortion = new StringBuffer( 10 );
        char ch;
        int i;
        for( i = 0; i < actualValue.length(); i++ )
        {
            ch = actualValue.charAt( i );
            if ( ch == '-' || ch == '.' || Character.isDigit( ch )) {
                numericPortion.append( ch );
            }
            else {
                break;
            }
        }

        // if actualValue == inputStr, no macro for units was found. We now know
        // how many chars of this inputStr were numeric, so we note this.
        if ( actualValue.equals( inputStr )) {
            ret.setCharsParsed( i );
        }
        else {
            ret.setCharsParsed( charsScanned );
        }

        float value;
        try {
            value = Float.parseFloat( numericPortion.toString() );
        }
        catch ( NumberFormatException nfe ) {
            return( ret );
        }

        // store the value in ret. then...
        ret.setAmount( value );


        // now figure out how many chars are in the unit abbrev
        int charsInUnitAbbrev = 0;
        for ( int j = i; j < actualValue.length(); j++ )
        {
            ch = actualValue.charAt( j );
            if ( ch == '}' || ch == ']' || ch == '|' ) {
                break;
            }
            else {
                charsInUnitAbbrev++;
            }                
        }
        ret.setCharsParsed( ret.getCharsParsed() + charsInUnitAbbrev );

        // get the type from what's left to parse of inputStr
        PlatyConst.unit unit = parseUnitType( actualValue.substring( i ));
        ret.setUnit(  unit );
        if ( unit == PlatyConst.unit.ERROR ) {
            ret.setErrorCode( Status.INVALID_UNIT  );
        }
        else {
            ret.setErrorCode( Status.OK );
        }

        return( ret );
    }

    /**
     * Gets the unit type. First it looks up any macro string, then converts the abbreviation for
     * unit to a unit type
     * @param inputStr the string that starts with the unit type
     * @return a ParsedUnit containing only type information.
     */
    public static PlatyConst.unit parseUnitType( final String inputStr )
    {
        String macroName;
        String actualValue;

        if ( inputStr == null ) {
            return( PlatyConst.unit.ERROR );
        }

        if ( inputStr.length() == 0 ) {
            return( PlatyConst.unit.NONE );
        }

        // test for string macro only (unit type cannot be numeric)
        if ( inputStr.charAt( 0 ) == '$' ) {
            macroName = Platypus.macros.extractMacroName( inputStr );
            actualValue = Platypus.macros.get( macroName );
            if ( actualValue == null ) {
                return( PlatyConst.unit.ERROR );
            }
        }
        else {
            actualValue = inputStr;
        }

        // at this point, we have the string containing the unit type with all macros resolved
        if ( actualValue.length() == 0 ) {
            return( PlatyConst.unit.NONE );
        }
        else {
            return( unitAbbrevToUnitType( actualValue ));
        }
    }

    /**
     * Convert the abbreviation used for the unit into a unit-type (an enum in PlatyConst)
     * @param unitStr the string that begins with the unit abbreviation
     * @return the PlatyConst.unit enum that corresponds to the type or to an error
     */
    public static PlatyConst.unit unitAbbrevToUnitType( final String unitStr )
    {
        char ch1;
        char ch2;

        if ( unitStr == null ) {
            return( PlatyConst.unit.ERROR );
        }

        if ( unitStr.length() == 0 ) {
            return( PlatyConst.unit.NONE );
        }
        
        ch1 = unitStr.charAt( 0 );
        if ( ch1 == '}' || ch1 == ']' || ch1 == '|' ) {
            return( PlatyConst.unit.NONE );
        }

        // if there is a unit specified, then it must have two letters. Get the second one.
        try {
            ch2 = unitStr.charAt( 1 );
        }
        catch( IndexOutOfBoundsException iobe ) {
            return( PlatyConst.unit.ERROR );
        }

        if ( ch1 == 'c' && ch2 == 'm' ) {
            return( PlatyConst.unit.CENTI );
        }
        else
        if ( ch1 == 'i' && ch2 == 'n' ) {
            return( PlatyConst.unit.INCH );
        }
        else
        if ( ch1 == 'l' && ch2 == 'i' ) {
            return( PlatyConst.unit.LINE );
        }
        else
        if ( ch1 == 'p' && ch2 == 't' ) {
            return( PlatyConst.unit.POINT );
        }
        else
        if ( ch1 == 'p' && ch2 == 'x' ) {
            return( PlatyConst.unit.PIXEL );
        }
        else
            return( PlatyConst.unit.ERROR );
    }

    /**
     * Static function that accepts a parameter and checks it for matching initial/closing { } 's.
     * Command parameters in Platypus that contain spaces are enclosed in { } characters. If the
     * parameter should contain a {, then multiple { 's can be used to open and close the parameter.
     * This function counts the number of opening {, builds a matching string of } 's and searches
     * for it in the parameter; it then returns whatever it finds between these characters (trimmed
     * of any white space. So: [ff:{times roman}] returns "times roman". Likewise {{{times romam}}}
     * @param rawParam the parameter from which we want to extract the data
     * @return the extracted data trimmed of leading/trainling white space; or null in case of error
     */
    public static String extractParameter( final String rawParam )
    {
        int start = 0;
        int end = 0;
        int count = 0;

        /** pattern of closing } characters that we're looking for at end of parameter */
        StringBuffer closePattern = new StringBuffer( 10 );

        if ( rawParam == null ) {
            return( null );
        }

        if ( rawParam.length() == 0 ) {
            return( null );
        }

        char rawParamChars[] = rawParam.trim().toCharArray();
        if( rawParamChars.length == 0 ) {
            return( "" );
        }

        if ( rawParamChars[0] == '{' ) {
            for( start = 0; rawParamChars[start] == '{' && start < rawParamChars.length; start++ )
                ;   // count the number of initial { characters

            count = start;
            while ( count-- > 0 ) {
                closePattern.append( '}' );
            }

            end = rawParam.trim().indexOf( closePattern.toString(), start );
            if ( end == -1 ) {
                return( null );
            }

            final String param = rawParam.trim().substring( start, end );
            return( param.trim() );
        }
        else {
            return( rawParam.trim() );
        }
    }
}
