 /*
  *  (c) Copyright 2006-07 Pacific Data Works LLC. All Rights Reserved.
  *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
  *
  *  For more information about platter, go to http://platypus.pz.org
  */

package platypus;
 
 /**
  * Immutable constants used in Platypus (Does not include status and error codes,
  * which are located in Status.java)
  * 
  * @author alb
  */
public final class PlatyConst
{
    /** must be at least two characters in a token */
    public static final int     MIN_TOKEN_LEN   =   2;

    /** units of measure */
    public enum unit {
        CENTI,          // centimeter
        INCH,
        LINE,
        NONE,           // no unit specified
        PIXEL,          // PIXEL not currently supported
        POINT,
        ERROR           
    }

    //=== text alignment ===//
    public static final Alignment.type DEFAULT_ALIGNMENT = Alignment.type.LEFT;

    //=== font constants ===/

    public static final float   DEFAULT_FONT_SIZE_PTS           =  12.0f;
    public static final float   DEFAULT_FOOTER_FONT_SIZE_PTS    =  10.0f;
    public static final String  DEFAULT_TYPEFACE                = "TIMES_ROMAN";
//    public static final String  DEFAULT_FOOTER_TYPEFACE         = "SYMBOL";
    public static final String  DEFAULT_FOOTER_TYPEFACE         = "TIMES_ROMAN";
    public static final float   MINIMUM_FONT_SIZE               =   4.0f;
    public static final float   DEFAULT_STRIKETHRU_LINE_WIDTH   =   0.5f;
    public static final float   DEFAULT_UNDERLINE_LOCATION      =  -2.0f;
    public static final float   DEFAULT_UNDERLINE_THICKNESS     =   0.6f;
    public static final float   LEADING_TO_FONT_RATIO           =   1.2f;
    public static final float   DEFAULT_LEADING_PTS             =
                                    DEFAULT_FONT_SIZE_PTS * LEADING_TO_FONT_RATIO;
    
    /** unless otherwise stated, strikethrough location is 1/4 of font size above the baseline */
    public static final float   FONT_SIZE_TO_STRIKETHRU_RATIO   =   4.0f;

    //=== paragraph defaults ===/

    public static final float  DEFAULT_PARA_INDENT_PTS          =   0.0f;

    //=== page size defaults ===/
    
    public static final float DEFAULT_PAGE_HT_PTS               = 792.0f;
    public static final float DEFAULT_PAGE_WD_PTS               = 612.0f;
    public static final float DEFAULT_MARGIN_PTS                =  72.0f;

    //=== layout defaults ===/
    public static final int   DEFAULT_NUMBER_OF_COLUMNS         =     1;

    // === other constants === /
    public static final int   DEFAULT_FOOTER_PAGES_TO_SKIP      =     1;

    //=== conversion constants for units of measure ===//

    /** one inch = 72 points */
    public static final float POINTS_PER_INCH   = 72.0f;

    /** one inch = 2.54 centimeters */
    public static final float CMS_PER_INCH      = 2.54f;

    /** one cm = 10mm */
    public static final float MMS_PER_CM        = 10.0f;


    //=== converstion routines for units of measure ===//

    public static float inchesToPoints( final float inches )
        { return inches * POINTS_PER_INCH; }

    public static float inchesToPoints( final int inches )
        { return inches * POINTS_PER_INCH; }

    public static float pointsToInches( final float points )
        { return points / POINTS_PER_INCH; }

    public static float cmsToPoints( final float cms )
        { return (( cms / CMS_PER_INCH ) * POINTS_PER_INCH ); }

    public static float cmsToPoints( final int cms )
        { return (( cms / CMS_PER_INCH ) * POINTS_PER_INCH ); }

    /**
     * Converts lines to points. A line is defined as current font size * LEADING_TO_FONT_RATIO.
     * (The temptation to make 1 line = to the current leading fails in the event that a user asks
     * for double space (2li). If this new leading becomes the new equivalent of a single line,
     * then the user has no way to go from this double-space back to single-space.)
     *
     * @param lines
     * @return points
     */
    public static float linesToPoints( final float lines )
        { return lines * GDD.getInstance().getCurrFont().getSize() * LEADING_TO_FONT_RATIO; }

    public static float linesToPoints( final int lines )
        {  return( linesToPoints( lines * 1.0f )); }


    /**
     * converts type and value to points
     * @param unit an enum representing the type of unit (inch, point, pixel, etc.)
     * @param amount the number of units
     * @return the number of points, or Float.NaN if the unit is invalid
     */
    public static float convertUnitsToPoints( final PlatyConst.unit unit, final float amount )
    {
        if ( unit == PlatyConst.unit.POINT || unit == PlatyConst.unit.NONE ) {
            return( amount );
        }

        if ( unit == PlatyConst.unit.INCH ) {
            return( PlatyConst.inchesToPoints( amount ));
        }
        else
        if ( unit == PlatyConst.unit.CENTI ) {
            return( PlatyConst.cmsToPoints( amount ));
        }
        else
        if ( unit == PlatyConst.unit.LINE ) {
            return( PlatyConst.linesToPoints( amount ));
        }
        else {
            // invalid unit type, (likely, it's unit.ERROR),
            // so return an impossible value
            return( Float.NaN );
        }
    }   
}
