/**
 * (c) Copyright 2007 Pacific Data Works LLC. All Rights Reserved.
 * Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus;

/*
    Change typeface: [ff:TIMES_ROMAN]
    Change size      [fsize:12pt]
    Change attribute:[+b][-b]; [+i][-i]; [f:normal]
 */

import com.lowagie.text.Font;
import com.lowagie.text.FontFactory;
import com.lowagie.text.pdf.BaseFont;

/**
 * PlatyFont contains font information that can be used in Platypus
 * It's not called Font, due to naming conflict with iText.
 */
public class PlatyFont
{
    /** an iText font data item */
    private Font font = null;

    /** line number in the input file at which this typeface was specified */
    private int lineNumber = 0;

    /** is font bold? */
    private boolean bold = false;

    /** is font italic/slanted? */
    private boolean italic = false;

    /** name of typeface as used by iText */
    private String typeface = PlatyConst.DEFAULT_TYPEFACE;

    /** size of font in points */
    private float size = PlatyConst.DEFAULT_FONT_SIZE_PTS;

    /** number of font directories registered */
    private int registeredFontDirectories = 0;

    
    /**
     * set up PlatyFont. Default to 12pt Times Roman, no bold or italics.
     */
    public PlatyFont()
    {
        // iText function that checks the standard places for fonts and registers them.
        // Because it's a very slow function, we make sure to run it only once.
        if ( registeredFontDirectories == 0 ) {
            registeredFontDirectories = FontFactory.registerDirectories();
        }

        setToDefault();
    }

    /**
     * set up PlatyFont. If only the font name is specified, use it with the current font size,
     * and current font attributes.
     * @param fontName the specified font name/typeface
     */
    public PlatyFont( final String fontName )
    {
        lineNumber = GID.getInstance().getCurrLine().getLineNumber();
        typeface = fontName;
        size = GDD.getInstance().getCurrFont().getSize();
        bold = GDD.getInstance().getCurrFont().isBold();
        italic = GDD.getInstance().getCurrFont().isItalic();
        font = setupFont();
    }

    /**
     * set up PlatyFont, using the current font but changing its size
     * @param fontSize the new font size
     */
    public PlatyFont( final float fontSize )
    {
        lineNumber = GID.getInstance().getCurrLine().getLineNumber();
        typeface = GDD.getInstance().getCurrFont().getTypeface();
        if ( fontSize != GDD.getInstance().getCurrFont().getSize() ) {
            setSize( fontSize );
        }
        bold = GDD.getInstance().getCurrFont().isBold();
        italic = GDD.getInstance().getCurrFont().isItalic();
        font = setupFont();
    }

    /**
     * set up PlatyFont using the specified font name and font size
     * @param fontName name of font to use
     * @param fontSize   size in points
     */
    public PlatyFont( final String fontName, final float fontSize )
    {
        lineNumber = GID.getInstance().getCurrLine().getLineNumber();
        typeface = fontName;
        if ( fontSize != GDD.getInstance().getCurrFont().getSize() ) {
            setSize( fontSize );
        }
        bold = GDD.getInstance().getCurrFont().isBold();
        italic = GDD.getInstance().getCurrFont().isItalic();
        font = setupFont();
    }

    /**
     * If pure, then set up a de minimus font with all ornaments turned off
     * @param pure
     * @param fontName
     * @param fontSize
     */
    public PlatyFont( boolean pure, String fontName, float fontSize )
    {
        if ( pure ) {   // TODO: do the right thing if not pure.
            lineNumber = GID.getInstance().getCurrLine().getLineNumber();
            typeface = fontName;
            size = fontSize;
            bold = false;
            italic = false;
            font = setupFont();
        }
    }

    /**
     * Does the actual heavy lifting of setting up the new font. All the parameters
     * that are needed are in the instance variables.
     * @return  the new Font object
     */
    public Font setupFont()
    {
        final Font f;
        final String iTextFontName;
        int iTextStyle = Font.NORMAL;

        // the 14 base fonts have to be treated differently than all other fonts.
        if ( typeface.equals( "COURIER" )   ||
             typeface.equals( "DINGBATS" )  ||
             typeface.equals( "HELVETICA" ) ||
             typeface.equals( "SYMBOL" )    ||
             typeface.equals( "TIMES_ROMAN" )) {
            return( handleBase14Font() );
        }

        // get the typeface

        if ( FontFactory.isRegistered( typeface )) {
                iTextFontName = typeface;
        }
        else {
             iTextFontName = PlatyConst.DEFAULT_TYPEFACE;
        }

        // new: get the size
        if ( size <= 0 ) {
            size = GDD.getInstance().getCurrFont().getSize();
        }

        // new: get the style

        if ( italic && bold ) {
            iTextStyle =  Font.BOLDITALIC;
        }
        else
        if ( italic ) {
            iTextStyle = Font.ITALIC;
        }
        else
        if ( bold ) {
            iTextStyle = Font.BOLD;
        }
        else {
            iTextStyle = Font.NORMAL;
        }

        // new: now make the font

        if ( FontFactory.isRegistered( iTextFontName )) {
            f = FontFactory.getFont( iTextFontName, BaseFont.CP1252, BaseFont.EMBEDDED,
                    size, iTextStyle );
            return( f );
        }
        else {
            GDD.getInstance().getLogger().warning(
                Literals.getInstance().getLit( "ERR_INVALID_FONT" ) + " " +
                typeface + " " +
                Literals.getInstance().getLit( "LINE_NUM_SIGN" ) + " " +
                GID.getInstance().getCurrLine().getLineNumber() );
            return font;
        }
    }

    /**
     * The base 14 postscript fonts are guaranteed in every version of Acrobat Reader.
     * iText treats them slightly differently from other fonts. This processing is handled
     * here.
     *
     * @return  the iText font corresponding to the correct base 14 font.
     */
    public Font handleBase14Font()
    {
        final BaseFont baseF;
        final Font f;
        String iTextFontName = null;

        // handle the different versions of base fonts, incl. typeface and font attributes.
        if ( typeface.equals( "COURIER" )) {
            if ( bold )
            {
                if ( italic )
                    iTextFontName = BaseFont.COURIER_BOLDOBLIQUE;
                else
                    iTextFontName = BaseFont.COURIER_BOLD;
            }
            else
            if ( italic )
                iTextFontName = BaseFont.COURIER_OBLIQUE;
            else
                iTextFontName = BaseFont.COURIER;
        }
        else
        if ( typeface.equals( "HELVETICA" )) {
            if ( bold )
            {
                if ( italic )
                    iTextFontName = BaseFont.HELVETICA_BOLDOBLIQUE;
                else
                    iTextFontName = BaseFont.HELVETICA_BOLD;
            }
            else
            if ( italic )
                iTextFontName = BaseFont.HELVETICA_OBLIQUE;
            else
                iTextFontName = BaseFont.HELVETICA;
        }
        else
        if ( typeface.equals( "TIMES_ROMAN" )) {
            if ( bold )
            {
                if ( italic )
                    iTextFontName = BaseFont.TIMES_BOLDITALIC;
                else
                    iTextFontName = BaseFont.TIMES_BOLD;
            }
            else
            if ( italic )
                iTextFontName = BaseFont.TIMES_ITALIC;
            else
                iTextFontName = BaseFont.TIMES_ROMAN;
        }
        else
        if ( typeface.equals( "SYMBOL" )) {
            iTextFontName = BaseFont.SYMBOL;
        }
        else
        if ( typeface.equals( "DINGBATS" )) {
            iTextFontName = BaseFont.ZAPFDINGBATS;
        }

        try
        {
            baseF = BaseFont.createFont( iTextFontName, BaseFont.CP1252,BaseFont.EMBEDDED);
        }
        catch( Exception e )
        {
            GDD.getInstance().getLogger().warning(
                Literals.getInstance().getLit( "ERR_INVALID_FONT" ) + " " +
                typeface + " " +
                Literals.getInstance().getLit( "LINE_NUM_SIGN" ) + " " +
                GID.getInstance().getCurrLine().getLineNumber() );
            return( null );
        }

        if ( size <= 0 ) {   // if = 0, use current font size
            size = GDD.getInstance().getCurrFont().getSize();
        }

        f = new Font( baseF, size );

        return( f );
    }

    /**
     * Sets a font's values to the defaults at the beginning of a document run. 
     */
    public void setToDefault()
    {
        font = setupFont();
        lineNumber = 0;
        typeface = PlatyConst.DEFAULT_TYPEFACE;
        size = PlatyConst.DEFAULT_FONT_SIZE_PTS;
        bold = false;
        italic = false;
        font = setupFont();
    }

    public void setBold( final boolean boldOnOff )
    {
        bold = boldOnOff;
    }

    public boolean isBold()
    {
        return( bold );
    }

    public void setItalic( final boolean italicOnOff )
    {
        italic = italicOnOff;
    }
    
    public boolean isItalic()
    {
        return( italic );
    }

    public void setFont( final Font newFont )
    {
        if ( font != null ) {
            font = newFont;
        }
    }
    /**
     * return the core font element in PlatyFont
     * @return the font object in PlatyFont
     */
    public Font getFont()
    {
        return( font );
    }

    public void setLineNumber( final int newLineNumber )
    {
        lineNumber = newLineNumber;
    }

    public int getLineNumber()
    {
        return( lineNumber );
    }

    public float getSize()
    {
        return( size );
    }

    public void setSize( final float newSize )
    {
        size = newSize;
        adjustStrikethru();
    }

    public void setTypeface( String newTypeface )
    {
        typeface = newTypeface;
    }
    
    public String getTypeface()
    {
        return( typeface );
    }

    /**
     * when the size of a font changes, the place where the strikethru line appears
     * should change, so that on a relative basis it crosses through the same part of
     * each character, regardless of the height of the character.
     */
    private void adjustStrikethru()
    {
        if ( GDD.getInstance().getStrikethru().isActive() ) {
            GDD.getInstance().getStrikethru().setAmountInPoints(
                    size / PlatyConst.FONT_SIZE_TO_STRIKETHRU_RATIO );
        }
    }
}
