/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-07 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.logging.Level;

import com.lowagie.text.FontFactory;

@SuppressWarnings("unchecked")  // don't issue spurious unchecked warnings on tree accesses
                                // works only with Java 5 compiler and later releases
/**
 * Main module for Platypus
 *
 * @author alb (Andrew Binstock)
 */
public class Platypus
{
    
    /** resource bundle that holds all the literals used in Platypus (singleton) */
    public static Literals lits;

    /** GDD: global document data: all data re the output file at any given point (singleton) */
    public static GDD gdd;

    /** GID: global input data: all data re the input file at any given moment (singleton) */
    public static GID gid;

    /** Platypus system strings. They are user-accessible as read-only */
    public static SystemStrings platyStrings;
    
    /** user-defined strings, numerics, and macros */
    public static UserString macros;

    /** factory for output-format-dependent classes */
    public static OutputFormatObjectFactory formatFactory = new OutputFormatObjectFactory();

    public Platypus()
    {
        initialize();
    }
    
    /**
     * Creates the global data blocks, gets the literals resource bundle,
     * sets up our default logger, and finally creates the data structures
     * for holding the system and user strings.
     */
    private static void initialize() 
    {
        // Get handles to the global document data blocks
        gid = GID.getInstance();
        gdd = GDD.getInstance();

        // Get the default resource bundle (US English strings) and load it.
        lits = Literals.getInstance();

        // Set up the primary logger and give it our own formatting
        gdd.setupLogger();

        // set up the table for system strings and for user macros
        platyStrings = new SystemStrings();
        macros = new UserString();
    }

    /**
     * Processes the command line.
     * @param args The command-line args
     * @return if no error, the number of args processed; otherwise, an error code.
     */
    public static int processCommandLine( String[] args )
    {
        if ( args == null ) {
            showUsage();
            showPleaseRerunMessage();
            return( Status.INVALID_PARAM_NULL );
        }

        if ( args.length == 0 ) {
            showUsage();
            showPleaseRerunMessage();
            return( Status.MISSING_PARAM );
        }

        if ( args.length == 1 ) {
            if ( processCommandLineSwitches( args ) != Status.COMPLETE ) {
                showUsage();
                showPleaseRerunMessage();
                return( Status.MISSING_PARAM );
            }
            else {
                return( Status.COMPLETE );
            }
        }

        // copy command line to table of Platypus system strings
        addCommandLineToPlatyStrings( args );

        // Start with the first two arguments: infile-name and outfile-name
        // Create an Infile instance for the input file and validate file's existence/readability.
        final Infile inf = new Infile( args[0] );
        if ( ! inf.doesExist() ) {
            gdd.getLogger().severe( lits.getLit( "ERR_FILE_MISSING1" ) + " " + args[0] + ". " +
                    lits.getLit( "ERR_FILE_MISSING2" ));
            return( Status.FILE_NOT_FOUND_ERR );
        }

        if ( inf.isReadable() ) {
            gid.setInfile( inf );
        }
        else {
            gdd.getLogger().severe( lits.getLit( "ERR_READING_FILE" ) + " " + args[0] + ". " +
                               lits.getLit( "ERR_FATAL_EXITING" ));
            return( Status.FILE_NOT_READABLE_ERR );
        }
        
        // set up output file: The file is actually opened later when we start sending data
        // to it. Right now, we just want to capture the name and identify the file format
        // from the extension. The actual Outfile class is generated by the formatFactory
        // class, which creates and returns various classes that depend on the output format.

        GDD.FileType fileType = identifyOutputFileType( args[1] );        
        
        if ( fileType != null ) {
            gdd.setFileTypeDependentClasses( fileType );
            gdd.getOutfile().setFilename( args[1] );
            gdd.getOutfile().setFileType( fileType );
        }
        else {
            gdd.getLogger().severe( lits.getLit( "ERR_OUTFILE_FORMAT" ));
            return( Status.ERR_UNSUPPORTED_FORMAT );
        }

        // process the remaining command-line parameters, if any
        if ( args.length > 2 ) {
            return( processCommandLineSwitches( args ));
        }
        else {
            return( 2 );
        }
    }

    /**
     * determine the file type of the output file
     * @param filename name of the output file (from which we determine format)
     * @return the FileType, or null on error
     */
    public static GDD.FileType identifyOutputFileType( String filename )
    {
        GDD.FileType ftype;
        String fname;

        if ( filename == null ) {
            return( null );
        }
        else {
            fname = filename.toLowerCase();
        }
                         
        if ( fname.endsWith( ".pdf" )) {
            ftype = GDD.FileType.IS_PDF;
            platyStrings.add( "$_outputFormat", "PDF" );
        }
        else
        if ( fname.endsWith( ".html") || fname.endsWith( ".htm" )) {
            ftype = GDD.FileType.IS_HTML;
            platyStrings.add( "$_outputFormat", "HTML" );
        }
    /*  .doc is not yet supported
        else
        if ( fname.endsWith( ".doc" )) {
            ftype = GDD.FileType.IS_DOC;
            platyStrings.add( "$_outputFormat", "MSWord" );
        } */
        else {
            ftype = null;
        }
        return( ftype );
    }

    /**
     * add command line to table of Platypus system strings
     * @param args the command-line arguments
     */
    private static void addCommandLineToPlatyStrings( String[] args )
    {
        StringBuffer sb = new StringBuffer( 50 );

        for( String arg : args )
        {
            sb.append( arg )
              .append( " ");
        }

        String commandLine = sb.toString();
        platyStrings.add( "$_commandLine", commandLine.trim() );
    }

    /**
     * Prints the usage message to the console, unless gdd.logger is set to no messages
     * (which only occurs in unit testing)
     */
    private static void showUsage()
    {
        /* we test for Logger set to OFF only because it appears in some unit tests where
         * we don't want program output to be shown.
         */
        if ( gdd.getLogger().getLevel() != Level.OFF)
            System.out.println( lits.getLit( "USAGE" ));
    }

    /**
     * The main processing routine. Processes the input file, closes the file and optionally
     * dumps the GDD
     * 
     * @return Status.OK in all cases. Eventually, should have greater variation, perhaps.
     */
    public static int mainProcessing()
    {
        /** status of operations */
        int status = Status.OK;

        System.out.println( lits.getLit( "PROCESSING" ) + " " + gid.getInfile().getFilename() +
                            lits.getLit( "THREE_DOTS" ) + "\n" );

        try {
            readInputFile();
            processInputFile();
        }
        catch( IOException e ) {
            System.out.println( e );
            status = Status.FATAL_ERR;
        }
        
        if ( gdd.getOutfile().isOpen() ) {
            gdd.getOutfile().close();
        }

        if ( gdd.isClVerbose() ) {
            gdd.dumpGDD();
        }

        return( status );
    }

    /**
     * processes all the items on the command line, other than the filenames of
     * the input and output files.
     *
     * @param args  the complete command line as specified by the user
     * @return number of args, if all OK; else Status.INVALID_PARAM
     */
    public static int processCommandLineSwitches( final String[] args )
    {
        if ( args == null ) {
            return( Status.INVALID_PARAM );
        }

        if ( args[0] == null ) {
            return( Status.INVALID_PARAM );
        }
        // some command lines that simply request information have only one
        // command-line argument.


        if ( args.length == 1 ) {
            if ( args[0].equalsIgnoreCase( "-help" )) {
                processClSwitch_help();
                return( Status.COMPLETE );
            }
            else
            if ( args[0].equalsIgnoreCase( "-listfonts" )) {
                processClSwitch_listfonts();
                return( Status.COMPLETE );
            }
            else {
                return( Status.INVALID_PARAM );
            }
        }

        // if there are two or more command-line args, the first two are the input and output
        // files. So, the following code handles args beyond those first two, if any.
        for ( int i = 2; i < args.length; i++ )
        {
            if ( args[0].equalsIgnoreCase( "-help" )) {
                processClSwitch_help();
                return( Status.COMPLETE );
            }
            else
            if ( args[i].equalsIgnoreCase( "-listfonts" )) {
                processClSwitch_listfonts();
                return( Status.COMPLETE );
            }
            else
            if ( args[i].equalsIgnoreCase( "-psize" )) {  // page size
                if ( processClSwitch_psize( args, i ) == Status.INVALID_PARAM ) {
                    return( Status.INVALID_PARAM );
                }
                else {
                    i++;    // show that we've processed the arg after -psize
                }
            }
            else
            if ( args[i].equalsIgnoreCase( "-verbose" )) {
                processClSwitch_verbose();
            }
            else
            if ( args[i].equalsIgnoreCase( "-vverbose" )) {
                processClSwitch_vverbose();
            }
            else {
                // on invalid arg: tell user and get out.
                gdd.getLogger().warning( lits.getLit( "ERR_COMMAND_LINE_IGNORED" ) + " " + args[i] );
                return( Status.INVALID_PARAM );
            }
        }
        return( args.length );
    }

    /**
     * show usage in response to -help command-line switch
     */
    private static void processClSwitch_help()
    {
        showUsage();
    }

    /**
     * print out message that appears after usage info when an error has occurred.
     */
    private static void showPleaseRerunMessage()
    {
        System.out.println( lits.getLit( "PLEASE_RERUN_EXITING" ));
    }

    /**
     * Writes the names of typefaces (as they should be used in the fontface command) that are
     * available in default directories on the current system. The names are then written to
     * stdout in alpha order.
     */
    private static void  processClSwitch_listfonts()
    {        
        ArrayList<String> typefaces = new ArrayList();

        // iText function that registers the available fonts on the runtime system
        FontFactory.registerDirectories();

        for( Iterator i = FontFactory.getRegisteredFamilies().iterator(); i.hasNext(); )
        {
            typefaces.add( (String) i.next() );
        }

        java.util.Collections.sort( typefaces );

        System.out.println( typefaces.size() + " " + lits.getLit( "DEFAULT_FONTS_ON_SYSTEM") +
                            "\n" );
        for( Iterator i = typefaces.iterator(); i.hasNext(); )
        {
            System.out.println( (String) i.next() );   
        }
    }

    /**
     * Process the -psize command-line switch ( = page size )
     * @param args the command-line arguments
     * @param i where -psize is in the array
     * @return Status.OK if all is well; Status.INVALID_PARAM if there's a problem
     */
    private static int processClSwitch_psize( String[] args, int i )
    {
        if ( ++i >= args.length ) {
            // verify that another parameter was specified
            gdd.getLogger().warning( lits.getLit( "MISSING_VALUE_FOR_SWITCH" ) + " " +
                    args[--i] );
            return( Status.INVALID_PARAM );
        }
        else {
            try {
                gdd.getPageSize().initialize( args[i] );
            }
            catch( IllegalArgumentException e ) {
                gdd.getLogger().warning( e + " " + args[i-1] + " : " + args[i] );
                return( Status.INVALID_PARAM );
            }
        }
        return( Status.OK );
    }

    /**
     * Process the -verbose command-line switch ( = verbose output )
     */
    private static void processClSwitch_verbose()
    {
        gdd.setClVerbose( true );
        gdd.getLogger().setLevel( Level.INFO );
    }

    /**
     * Process the -vverbose command-line switch ( = very verbose output )
     * -vverbose implies -verbose
     */
    private static void processClSwitch_vverbose()
    {
        gdd.setClVerbose( true );
        gdd.setClVVerbose( true );
        gdd.getLogger().setLevel( Level.ALL );
    }

    public static void readInputFile() throws IOException
    {
        // Read the contents of the file into an ArrayList of lines,
        // located in the Infile object, gid.getInfile().

        final int linesRead = gid.getInfile().readFile();
        if ( linesRead <= Status.ERR ) {
            gdd.getLogger().severe( lits.getLit( "ERR_READING_FILE" ) +
                               gid.getInfile().getFilename() + lits.getLit( "ERR_FATAL_EXITING" ));
            throw new IOException( lits.getLit( "ERR_FATAL_EXITING" ));
        }
        else
        if ( linesRead == 0 ) {
            // If file was empty...
            gdd.getLogger().severe( lits.getLit( "ERR_FILE_ERROR" ) + " " +
                                    gid.getInfile().getFilename() + " " +
                                    lits.getLit( "ERR_FILE_EMPTY" ));
            throw new IOException( lits.getLit( "ERR_FATAL_EXITING" ));
        }
        else {
            int bc = gid.getInfile().stripBlockComments();
            if ( gdd.isClVVerbose() ) {
                gdd.getLogger().finer( bc + " " + lits.getLit( "COMMENTS_FOUND_LINES_PROCESSED" ));
                for (ReadLine textLine : gid.getInfile().textLines)
                {
                    gdd.getLogger().finest( lits.getLit( "LINE_NUM_SIGN" ) + " " +
                        textLine.getLineNumber() + ": " + textLine.getLineContent() );
                }
            }
        }
    }

    /**
     * Process the file line-by-line: breaking the line into tokens and
     * then processing those.
     */
    public static void processInputFile()
    {
        while ( gid.getInfile().getLine() != Status.AT_EOF )
        {
            ReadLine currLine = gid.getCurrLine();
            gid.setCurrentLineTokens( new Tokens( currLine.parse() ));
            gid.getCurrentLineTokens().process();
            gid.getInfile().incrementLineCount();
        }
    }

    /**
     * This mainline:
     * 1) Sets up the major data structures
     * 2) Writes the version number and copyright notice to the console
     * 3) the command line is processed
     * 4) the main processing is performed.
     *
     * @param args the command-line arguments
     */
    public static void main( String[] args )
    {
        // set up the main global data blocks
        initialize();

        // PlatyVersion holds a string with the current version number
        final PlatyVersion ver = new PlatyVersion();

        // print out the version number and copyright to the console
        System.out.println( "platypus v. " + ver.toString() + " " +
                            lits.getLit( "COPYRIGHT_NOTICE" ) + "\n" );

        // add the version number to the global strings table
        platyStrings.add( "$_version", ver.toString() );

        // the main processing
        try
        {
            int status = processCommandLine( args );
            if ( status != Status.COMPLETE && status >= Status.OK ) {
                mainProcessing();
            }
        }
        catch( Exception e )    // universal catch-all for any uncaught exceptions
        {
            gdd.getLogger().severe( lits.getLit( "ERR_UNEXPLAINED" ) + e );
            System.out.println( lits.getLit( "LINE_NUM_SIGN" ) +
                    gid.getCurrLine().getLineNumber() +  ": " +
                    gid.getCurrLine().getLineContent() );
        }
    }
}
