/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-07 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 *
 *  Note: eventually, when we add #include commands, we will need to keep track of which
 *        file is being read in addition to the line contents and line number.
 *
 */
package platypus;

import java.util.ArrayList;

/**
 * Defines a ReadLine, which simply holds the line (as a string) and
 * the line # from the imput file.
 *
 * @author alb
 */
public class ReadLine
{
    /** the content of the line as a string */
    private String lineContent;

    /** the line number */
    private int lineNumber;

    public ReadLine()
    {
        this.lineNumber = 0;
        this.lineContent = null;
    }

    public ReadLine( int number, String text )
    {
        this.lineNumber = number;
        this.lineContent = text;
    }

    /**
     * Parses the readline into an array of tokens.
     *
     * @return an ArrayList of parsed tokens. Errors are handled by values in the ArrayList.
     */
    public ArrayList<Token> parse()
    {
        int i;
        String line;
        ArrayList<Token> tokens = new ArrayList<Token>();

        tokens.add( new Token( TokTyp.LINE_NUMBER,
                               Integer.toString( GID.getInstance().getCurrLine().lineNumber )));
        line = lineContent;
        while( true )
        {
            // if there are no more tokens to process in line, then write it out and return.
            i = line.indexOf( '[' );
            if ( i == -1 )
            {
                tokens.add( new Token( TokTyp.TEXT, line ));
                break;
            }

            // check for all the cases in which a [ would NOT be a command start. These are:
            //    1) there is a preceding / character (that is: /[ is a simple [ character)
            //    2) it's the last character in the line
            //    3) there is a digit immediately following the [ character
            //    4) there is whitespace immediately following the [ character
            // in all these cases write out everything upto and including the [ as text
            
            if (( i > 0 && line.charAt( i-1 ) == '/'  ) ||         // preceded by a / ?
                    i + 1 >= line.length() ||                      // last character in line?
                    Character.isDigit( line.charAt( i+1 )) ||      // followed by a digit?
                    Character.isWhitespace( line.charAt( i+1 )))   // followed by a whitespace?
            {
                    tokens.add( new Token(
                        //ouput everything upto and including [ (except a preceding / character)
                        TokTyp.TEXT,
                        line.substring( 0, (( i > 0 && line.charAt(i-1) == '/' ) ?
                                i-1 : i )).concat( "[" )));
                    line = line.substring( i+1 );
                    if ( line.length() > 0 )
                        continue;
                    else
                        break;

            }
            else
            {   // it's a command

                // first write out all preceding stuff as text; then remove
                // that leading material so that line begins with command's [
                if ( i > 0 )
                {
                    tokens.add( new Token( TokTyp.TEXT, line.substring( 0, i )));
                    line = line.substring( i );
                }
                i = 0;

                // now check for the token-ending ]
                int k;
                for ( k = i+1; k < line.length(); k++ )
                    if ( line.charAt( k ) == ']' && line.charAt( k-1 ) != '/' )
                        break;

                // we reached EOL w/out finding the token end. This is an error.
                if ( line.charAt( k ) != ']' )
                {
                    tokens.add( new Token( TokTyp.UNCLOSED_COMMAND, line ));
                    break;
                }

                // otherwise, right out the command, trim the line to whatever remains
                // and loop again looking for commands.
                tokens.add( new Token( TokTyp.COMMAND, line.substring( 0, k+1 )));
                line = line.substring( k+1 );
                if ( line.length() > 0 )
                    continue;
                else
                    break;
            }
        }

        for ( int j = 0; j < tokens.size(); j++ )
        {
            GDD.getInstance().getLogger().finer(
                "type: " + tokens.get( j ).getType() + " |> " + tokens.get( j ).getContent() );
        }

        return( tokens );
    }

    /**
     * return the line number we're currently working on
     * @return lineNumber
     */
    public int getLineNumber()
    {
        return( lineNumber );
    }

    /**
     * return a string with the line content
     * @return lineContent
     */
    public String getLineContent()
    {
        return( lineContent );
    }

    public void setContent( final String newContent )
    {
        lineContent = newContent;
    }

    /**
     * Set line number
     * @param newNumber the new line number
     */
    public void setLineNumber( final int newNumber )
    {
        lineNumber = newNumber;
    }
}
