/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-08 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */
package platypus;

import java.util.ArrayList;

/**
 * Convenience class for handling the array of tokens (that constitute an input line)
 * @author alb
 */
public class Tokens
{
    /** the ArrayList of tokens that have been parsed */
    private ArrayList<Token> tokens;

    private final GDD gdd = GDD.getInstance();
    private final GID gid = GID.getInstance();
    private final Literals lits = Literals.getInstance();

    public Tokens( final ArrayList<Token> toks )
    {
        tokens = toks;
    }

    /**
     * Process the tokens in the array of tokens for a single input line,
     * at a basic level:
     * 
     * line number, text, and commands are all dispatched to the
     * routines that process them further.
     *
     * @return Status.OK if all is well, otherwise an error status.
     */
    public int process()
    {
        if ( tokens == null ) {
            return( Status.ERR );
        }
        // verify that every line to be processed begins with a line #
        if ( tokens.get( 0 ).getType() != TokTyp.LINE_NUMBER ) {
            gdd.getLogger().severe( lits.getLit( "ERR_LINE_WITHOUT_LINE_NUMBER" ) + " " +
                                gid.getCurrLine().getLineNumber()  +  "." +
                                lits.getLit( "LINE_IGNORED" ) + "." );
            return( Status.INVALID_TOKEN );
        }

        // verify that there are at least two tokens in the line (line #, plus some content)
        if ( tokens.size() < 2 ) {
            gdd.getLogger().severe( lits.getLit( "ERR_MISSING_TOKEN_AT_LINE" ) + " " +
                                gid.getCurrLine().getLineNumber()  +  "." +
                                lits.getLit( "LINE_IGNORED" ) + "." );
            return( Status.MISSING_TOKEN );
        }

        // reset the global line number
        gid.getCurrLine().setLineNumber( -1 );

        gid.setCurrentTokenNumber( 0 );
        // dispatch the various tokens for processing, based on token type
        while ( gid.getCurrentTokenNumber() < tokens.size() )
        {
            Token t = tokens.get( gid.getCurrentTokenNumber() );
            // can only happen if an error occurred loading up Tokens. Should never occur.
            if ( t == null || t.getContent() == null ) {
                gdd.getLogger().severe( lits.getLit( "ERR_INVALID_TOKEN_LINE" ) + " "  +
                                     gid.getCurrLine().getLineNumber() );
                return( Status.INVALID_PARAM_NULL );
            }
            else {
                gid.setCurrentTokenNumber( gid.getCurrentTokenNumber() + 1 );
            }

            // dispatch the token
            switch( t.getType() )
            {
                case LINE_NUMBER:
                    gid.getCurrLine().setLineNumber( Integer.valueOf( t.getContent() ));
                    break;
                case TEXT:
                    gdd.getOutfile().emitText( t.getContent() );
                    break;
                case COMMAND:
                    gdd.getCommandProcessor().process( t.getContent().toCharArray() );
                    break;
                case UNCLOSED_COMMAND:
                    gdd.getLogger().warning( lits.getLit( "ERR_UNCLOSED_COMMAND" ) + " " +
                                         gid.getCurrLine().getLineNumber()  + " " +
                                         lits.getLit( "IGNORED_IS_TEXT"));
                    gdd.getOutfile().emitText( t.getContent() );
                    break;
                default:
                    gdd.getLogger().severe( lits.getLit( "ERR_INVALID_TOKEN_LINE" ) + " "  +
                                        gid.getCurrLine().getLineNumber()  +
                                        lits.getLit ( "IGNORED_TOKEN_TYPE" ) + t.getType() );
            }
        }
        return( Status.OK );
    }

    public void setTokens( ArrayList<Token> newTokens )
    {
        tokens = newTokens;
    }

    public ArrayList<Token> getTokens()
    {
        return( tokens );
    }
}
