/*--
 *(c) Copyright 2007 Pacific Data Works LLC. All Rights Reserved.
 * Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */
package platypus;
import java.util.TreeMap;

/**
 * Maintains a table of macros. Macros have the following properties:
 * Strings:
 *      Begin with a $ followed by a letter or digit and then
 *                                      any combination of letters, digits, and _
 *      Users can read, write, modify at will
 * Platypus-defined strings:
 *      Begin with $_ followed by any combination of letters, digits, and _ characters
 *      To the user, Platypus-defined strings are read-only
 * All string names are case-sentistive.
 *
 * Strings are defined with the [def:StringName{contents of string}]
 *      default is " " (a single space)
 *      strings cannot currently contain commands or other strings
 *
 * Numbers default to 0, implemented in BigDecimal, so no over/undeflow errors possible
 *
 * To print a string to the document, enter an asterisk before the string name: [*StringName]
 * Note: no translation is done of the string before it's printed.
 *
 * Additional info: Platypus will expose its own variables as read-only macros, as indicated
 * above.
 *
 * @author alb
 */
public class UserString
{
    @SuppressWarnings("unchecked")  // don't issue spurious unchecked warnings on tree accesses
                                    // works only with Java 5 compiler, later releases
    final TreeMap uStrings;

    /**
     * Create a table for key-value pairs (both strings)
     */
    public UserString()
    {
        uStrings = new TreeMap<String, String>();
    }

    /**
     * Add a macro to the table of user strings
     * @param key the macro name
     * @param value the macro body
     * @return Status.OK if all went well; Status.INVALID_PARAM on error
     */
    public int add( final String key , final String value )
    {
        if ( key == null || value == null ) {
            return( Status.INVALID_PARAM_NULL );
        }

        // user string must start with $ and cannot have _ as second character
        if (( key.charAt(0) != '$' ) || key.charAt(1) == '_' ) {
            return( Status.INVALID_PARAM );
        }           

        try {
            uStrings.put( key, value );
        }
        catch( NullPointerException e ) {
            return ( Status.INVALID_PARAM_NULL );
        }
        catch ( ClassCastException e ) {
            return ( Status.INVALID_PARAM );
        }

        return( Status.OK );    
    }

    /**
     * Retrieve a string based on its key (the macro name)
     * @param key  the macro name
     * @return the contents of the macro if found; otherwise, null
     */
    public String get( final String key )
    {
        if ( key == null ) {
            return( null );
        }

        Object o = uStrings.get( key );
        if ( o == null ) {
            return( null );
        }
        else {
            return o.toString();
        }
    }

    /**
     * Updates a value by deleting the current KV pair and replacing it with the new one
     * @param key new key
     * @param value new data/value
     * @return  Status.OK if all OK; Status.INVALID_PARAM or Status.INVALID_NULL_PARAM on error
     */
    public int update( final String key, final String value ) {
        if ( uStrings.containsKey( key )) {
            uStrings.remove( key );
            return( add( key, value ));
        }
        else {
            return( Status.INVALID_PARAM );
        }
    }

    /**
     * How many strings are there in the table?
     * @return number of strings
     */
    public int sizeof() {
        return( uStrings.size() );
    }

    /**
     * Dump all the strings in the table to one long string.
     * @return  string containing all the KV pairs, one per line, properly formatted
     */
    @Override
    public String toString() {
        return( dumpUserStrings() + "\n" + dumpPlatypusStrings() );
    }

    /**
     * Dump all the user-defined string to one long string.
     * @return string containing all the KV pairs, one per line, properly formatted
     */
    public String dumpUserStrings() {
        int counter = 0;
        final StringBuffer sb = new StringBuffer( sizeof() * 30 );
        Object content = null;

        sb.append( Literals.getInstance().getLit( "USER_DEFINED_STRINGS" ) + "\n" );
        
        for ( Object keyObj : uStrings.keySet())    //TODO: Combined with code in next method
        {
            if ( keyObj != null ) {
                if ( ! keyObj.toString().startsWith( "[$" )) { // don't want Platy Strings
                    content = uStrings.get( keyObj.toString() );
                    sb.append( "\t" + keyObj.toString() + ": " + content.toString() + "\n" );
                    counter++;
                }
            }
        }

        if ( counter == 0 ) {
            sb.append( "\t" + Literals.getInstance().getLit( "NONE" ));
        }
        return( sb.toString() );
    }

    /**
     * Dump all the strings defined by Platypus in the table to one long string.
     * @return  string containing all the KV pairs, one per line, properly formatted
     */
    public String dumpPlatypusStrings() 
    {
        return( Platypus.platyStrings.dump() ); 
//        int counter = 0;
//        final StringBuffer sb = new StringBuffer( sizeof() * 30 );
//
//        sb.append( Literals.getInstance().getLit( "PLATYPUS_STRINGS" ) + "\n" );
//
//        Set set = Platypus.platyStrings.entrySet();
//        Iterator iterator = set.iterator();
//        while ( iterator.hasNext() )
//        {
//            Map.Entry entry =  ( Map.Entry ) iterator.next();
//            sb.append( "\t" )
//              .append( entry.getKey() )
//              .append( ": " )
//              .append( entry.getValue() )
//              .append( "\n" );
//
//            counter++;
//        }
//
//        if ( counter == 0 ) {
//            sb.append( "\t" )
//              .append( Literals.getInstance().getLit( "NONE" ));
//        }
//        return( sb.toString() );
    }

    /**
     * Accepts a string that should start with the name of a Platypus string
     * and returns the portion that's a legal system macro name: $PageCount
     * Stops at the first invalid character
     * @param rawString the string that should begin with a system macro name
     * @return the actual string, or null if an error occurs
     */
    public String extractMacroName( final String rawString )
    {

        if ( rawString == null ) {
            return( null );
        }

        // must start with a $ or #
        char ch = rawString.charAt( 0 );
        if ( ch != '$' ) {
            return( null );
        }

        StringBuffer sb = new StringBuffer( 20 );
        sb.append( ch );

        for ( int i = 1; i < rawString.length(); i++ )
        {
            ch = rawString.charAt( i );
            if ( Character.isLetterOrDigit( ch ) || ch == '_' ) {
                sb.append( ch );
            }
            else {
                break;
            }
        }
        return( sb.toString() );
    }
}
