/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-07 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus;

/**
 *  Class that contains a value and the line number of its last modification.
 *  Uses generics. Note: because of the use of generics, getValue() returns a
 *  *string*. This should be converted to the right numeric type through the
 *  usual Java library methods.
 *  @author alb
 */
public class Value<A>
{
    /** the actual value (of unknown type) */
    private A value;

    /** the input line # at which value was set */
    private int lineNumber;

    public Value( final A initialValue )
    {
        value = initialValue;
        lineNumber = 0;
    }

    public Value( final A val, final int line )
    {
        value = val;
        lineNumber = line;
    }

    /**
     * In a parameterized class, this returns an Object even if declared as A getValue(), so instead
     * return a string and convert it at point of use via Float.valueOf(), Integer.valueOf(), 
     * Boolean.valueOf, etc.
     *
     * @return  value as a String
     */
    public String getValue()
    {
        return( value.toString() );
    }

    /**
     * Returns an integer containing the line number of when the value was last modified
     * @return the line number
     */
    public int getLineNumber()
    {
        return( lineNumber );
    }

    /**
     * Setter for value (but returns set value object as a convenience. Note: despite the use of
     * a generic for a return value, the returned item is of class Object
     * @param newValue the value to set value to.
     * @return an Object equal to the newValue.
     */
    public A setValue( final A newValue )
    {
        value = newValue;
        if ( GID.getInstance().getInfile() == null ) {
            lineNumber = 0;
        }
        else {
            lineNumber = GID.getInstance().getCurrLine().getLineNumber();
        }
        return( newValue );
    }

    /**
     * Creates a partial string containing the dump information for the value.
     * @return a string containing the line number in parens followed by the value.
     */
    public String dump()
    {
        StringBuffer sb = new StringBuffer();
        sb.append( "(" )
          .append( Literals.getInstance().getLit( "LINE" ))
          .append( " " )
          .append( lineNumber  )
          .append( ") " )
          .append( getValue() );
        return( sb.toString() );
    }
}
