/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-07 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus.commands;

import platypus.*;

import java.util.ArrayList;

/**
 * Takes a macro, looks it up, and parses it. The resulting tokens are placed in the current
 * readline at the spot of the next token so that they are processed. The macro may contain
 * commands, but it cannot contain additional macros. In other words, macros do not nest. At
 * least not for the time being.
 *
 * @author alb
 */
public class CommandParseMacro
{
    @SuppressWarnings("unchecked")  // don't issue spurious unchecked warnings on tree accesses
                                    // works only with Java 5 compiler, later releases

    public static int process( GDD gdd, GID gid, final Literals lits, final String macro )
    {
        // remove the leading [ and trailing ]
        String key = macro.substring( 1, macro.length()-1 );

        // look up the macro.
        String value = Platypus.macros.get( key );
        if ( value == null ) {
             return( Status.INVALID_TOKEN );
        }

        // create a ReadLine with the returned macro value, then parse it.
        ReadLine line = new ReadLine( gid.getCurrLine().getLineNumber(), value );
        ArrayList<Token> tokens = line.parse();

        // did we get valid tokens?
        if ( tokens == null ) {
            return( Status.INVALID_TOKEN );
        }
        if ( tokens.size() < 2 )  {
            return( Status.INVALID_TOKEN );
        }

        // the first token is a line number, so we remove this as we are already in a line.
        tokens.remove( 0 );

        for ( int i = 0; i < tokens.size(); i++ ) {
            Token token =  tokens.get( i );
            if ( token.getType() == TokTyp.COMMAND ) {
                if ( token.getContent().startsWith( "[$" )) {
                    gdd.getLogger().warning( lits.getLit( "(line# ") +
                                gid.getCurrLine().getLineNumber() + ") " +
                                lits.getLit( "macros cannot nest." ) + " " +
                                token.getContent() + " in " + macro + " ignored." );
                    tokens.remove( i );
                    i--; // so that the next cycle will look at the new current token
                }
            }

        }

        // now take the list of tokens and insert them in the current line at the present point
        // we do this by creating a new set of Tokens for the entire line (including the ones we've
        // already scanned and then replacing the current readline tokens with this one.
        if ( tokens.size() > 0 ) {
            final Tokens currLine = gid.getCurrentLineTokens();
            int currTokenNumber = gid.getCurrentTokenNumber();
            
            ArrayList newToks = new ArrayList<Token>( currTokenNumber * 2 );

            // first copy over the tokens that Platypus has processed so far in this line
            int i;
            for ( i = 0; i < currTokenNumber; i++ )
            {
                newToks.add( i, currLine.getTokens().get( i ));
            }

            // then add the new tokens we've just expanded
            for ( int j = 0; j < tokens.size(); j++ )
            {
                newToks.add( i, tokens.get( j ));
                i++;
            }

            // then add any remaining tokens from the original line, if any
            if ( currTokenNumber < currLine.getTokens().size() ) {
                for ( int j = currTokenNumber + 1; j < currLine.getTokens().size(); j++ )
                {
                    newToks.add( i, currLine.getTokens().get( j ));
                }
            }
 ///           toks.add( new Token( TokTyp.COMMAND, "[]" ) );
            gid.setTokens( new Tokens( newToks ));
//            final Tokens currLine = gid.getCurrentLineTokens();
//            final ArrayList<Token> currLineTokens = currLine.getTokens();
//            int currTokenNumber = currLine.getCurrTokenNumber();
//            currTokenNumber++;
//            for ( int i = 0; i < tokens.size(); i++ ) {
//                currLineTokens.add( currTokenNumber++, tokens.get( i ));
//            }
        }
        return( Status.OK );  
    }

}
