/**
 *  (c) Copyright 2006-7 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */
package platypus.html;

import platypus.*;

import java.io.FileWriter;
import java.io.IOException;

/**
 * Handles document creation/termination and command routines for output
 * to HTML
 * 
 * @author alb
 */
public class OutfileHtml extends Outfile
{
    private final GDD gdd;
    private final Literals lits;
    private FileWriter fout;
    private StringBuilder currParagraph;

    public OutfileHtml()
    {
        gdd = GDD.getInstance();
        lits = Literals.getInstance();
        setOpenStatus( false );
        currParagraph = null;
    }

    /**
     * opens HTML file and outputs the minimal starting text.
     *
     * Important note: This method is called only when Platypus has text
     * to emit, so we know something about state at this point: All commands
     * that must precede the first text have been processed, and there is
     * something to write to the output file.
     *
     * @return  Status.OK on normal completion; otherwise, an error code.
     */
    public int open()
    {
        PlatyVersion ver = new PlatyVersion();

        try {
            fout = new FileWriter( getFilename() );
            setOpenStatus( true );
        }
        catch ( IOException e ) {
            return( Status.FATAL_ERR );
        }

        emitString( "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" "+
                    "\"http://www.w3.org/TR/html4/loose.dtd\">\n" +
                    "<HTML>\n" +
                    "<HEAD>\n" +
                    "<!-- " +
                    lits.getLit( "PLATYPUS_CREATED_BY")
                    + ver.toString() +
                    "  " +
                    lits.getLit( "AVAILABLE_AT_PZ_ORG" ) +
                    "-->\n" +
                    "</HEAD>\n" +
                    "<BODY>\n" );

        setTextProcessedFlag( true );
        return( Status.OK );
    }

    /**
     * verifies the file is open, and if it's not, opens it.
     * Used by write routines.
     */
    public void verifyOpen()
    {
        if ( ! isOpen() ) {
            if ( open() != Status.NO_ERROR ) {
                gdd.getLogger().severe( lits.getLit( "ERR_OUTFILE_OPEN" ) + " " +
                        getFilename() + " " +
                        lits.getLit( "ERR_FATAL_EXITING" ));
                System.exit( Status.FATAL_ERR );
            }
        }
    }
    /**
     * close the current paragraph and open a new one
     */
    @Override
    public void doNewParagraph()
    {
        // if the current paragraph has already been flushed, (so, currParagaph = null ),
        // we don't do anything, otherwise we close the paragraph and set currParagaph to null.
        // This has the effect of forcing the creation of a new paragraph when new text is
        // written out.
        if ( currParagraph != null ) {
            emitString( currParagraph.toString() );
            emitString( "</p>\n" );
            currParagraph = null;
        }
    }

    /**
     * Two actions can result from an item in an input file: it can set the value of a
     * variable (such as margins, for example) or result in text that must be emitted to the
     * output file stream. emitText() handles this latter function.
     *
     * @param s text string to send to output
     */
    @Override
    public void emitText( String s )
    {
        if ( s == null ) {
            return; //TODO: log an error here
        }

        if ( s.length() == 0 ) {
            return; // TODO: log a warning here
        }

        // at this point, we know there's something to output. So, first we
        // make sure outpuf file is open, then we emit the token.  The first
        // time through, the file will be closed, and we'll open it from here.
        // This prevents us creating a file until we know for sure there is
        // something to write out.

        verifyOpen();

        // if there is no currently open paragraph, start one 
        if ( currParagraph == null ) {
            currParagraph = new StringBuilder();
            currParagraph.append(  "<p>\n");
        }

        currParagraph.append( s );
    }

    /**
     * Lower level that emitText() and higher level than emitString().
     * This use emitText() if we're in a paragraph and emitString() if we're not.
     * This enables us to write out formatting commands to an HTML file without
     * starting a new paragraph.
     * @param s
     */
    @Override
    public void emitRawText( String s )
    {
        verifyOpen();

        if ( currParagraph == null ) {
            emitString( s );
        }
        else {
            emitText( s );
        }
    }

    /**
     * Low level function for writing characters to HTML file
     * @param s string to write out
     * @return Status.OK if all went well, otherwise Staus.IO_ERR
     */
    protected int emitString( String s )
    {
        try {
            fout.write( s );
        }
        catch( IOException e ) {
            return( Status.IO_ERR );
        }

        return( Status.OK );
    }

    /**
     * Closes the file after flusing any remaining text and writing out the required
     * concluding HTML
     */
    @Override
    public void close()
    {
        // write out any text waiting to be written out
        doNewParagraph();

        // write out the concluding HTML
        emitString( "</body>\n" + "</HTML>" );

        // then flush the stream, and close the file.
        try {
            fout.flush();
            fout.close();
        }
        catch ( IOException e ) {
            //TODO: Do something smart here
        }
    }

    /**
     * Tells us whether we are in the middle of a paragraph or not
     * @return boolean true = in paragraph; false = not in one
     */
    public boolean inParagraph()
    {
        return( currParagraph == null ? false : true );
    }
}
