/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-07 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus.pdf;

import platypus.*;
import platypus.pdf.commands.*;
import platypus.pdf.commands.characters.*;

/**
 * Converts command strings from Platypus documents into actions.
 * It consists of putting all the classes that perform these actions
 * into the command table, where they will be loaded and run.
 *
 * @author alb
 */
public class CommandProcess extends CommandProcessor 
{
    public CommandProcess()
    {
        gdd = GDD.getInstance();

        initializeCommandTable();
    }

    /**
     * Load the individual commands supported by PDF into the command table.
     * These commands overwrite the default commands in the table, as it was
     * initialized in Platypus. (Default: emit warning that the command is not
     * supported.) Consequently, any commands not loaded here into the table,
     * will default to the unsupported-error message.
     */
    public void initializeCommandTable()
    {
        CommandTable ct = gdd.getCommandTable();
        loadCommands( ct );
        loadSpecialChars( ct );
    }

    /**
     * Load the commands other than those that ouptut symbols and special chars.
     *
     * @param ct the command table in which to load them
     */
    public void loadCommands( final CommandTable ct )
    {
        // commands in alpha order, except for symbols and foreign characters
        ct.load( new CommandAlignment() );
        ct.load( new CommandColumns() );
        ct.load( new CommandDump() );
        ct.load( new CommandFontBoldOff() );
        ct.load( new CommandFontBoldOn() );
        ct.load( new CommandFontFace() );
        ct.load( new CommandFontItalicsOff() );
        ct.load( new CommandFontItalicsOn() );
        ct.load( new CommandFontSize() );
        ct.load( new CommandLeading() );
        ct.load( new CommandMarginBottom() );
        ct.load( new CommandMarginLeft() );
        ct.load( new CommandMarginRight() );
        ct.load( new CommandMarginTop() );
        ct.load( new CommandMarginsMirrored() );
        ct.load( new CommandNewPage() );
        ct.load( new CommandNewParagraph() );
        ct.load( new CommandNoindent() );
        ct.load( new CommandPageHeight() );
        ct.load( new CommandPageNumber() );
        ct.load( new CommandPageNumberFormat() );
        ct.load( new CommandPageSize() );
        ct.load( new CommandPageWidth() );
        ct.load( new CommandParaIndent() );
        ct.load( new CommandParaSkip() );
        ct.load( new CommandStrikethruOff() );
        ct.load( new CommandStrikethruOn() );
        ct.load( new CommandUnderlineOn() );
        ct.load( new CommandUnderlineOff() );
    }

    /**
     * Load symbols and foreign characters in alpha letter name (lower case before uppper case)
     *
     * @param ct the command table in which to load them
     */
    public void loadSpecialChars( final CommandTable ct )
    {
        ct.load( new CharAAcute() );
        ct.load( new CharACircumflex() );
        ct.load( new CharAGrave() );
        ct.load( new CharAUmlaut() );
        ct.load( new CharACapAcute() );
        ct.load( new CharACapCircumflex() );
        ct.load( new CharACapGrave() );
        ct.load( new CharACapUmlaut() );
        ct.load( new CharCCedilla() );
        ct.load( new CharCCapCedilla() );
        ct.load( new CharEAcute() );
        ct.load( new CharECircumflex() );
        ct.load( new CharEGrave() );
        ct.load( new CharEmDash() );
        ct.load( new CharEnDash() );
        ct.load( new CharEUmlaut() );
        ct.load( new CharECapAcute() );
        ct.load( new CharECapCircumflex() );
        ct.load( new CharECapGrave() );
        ct.load( new CharECapUmlaut() );
        ct.load( new CharIAcute() );
        ct.load( new CharICircumflex() );
        ct.load( new CharIGrave() );
        ct.load( new CharIUmlaut() );
        ct.load( new CharICapAcute() );
        ct.load( new CharICapCircumflex() );
        ct.load( new CharICapGrave() );
        ct.load( new CharICapUmlaut() );
        ct.load( new CharNTilde() );
        ct.load( new CharNCapTilde() );
        ct.load( new CharOAcute() );
        ct.load( new CharOCircumflex() );
        ct.load( new CharOGrave() );
        ct.load( new CharOUmlaut() );
        ct.load( new CharOCapAcute() );
        ct.load( new CharOCapCircumflex() );
        ct.load( new CharOCapGrave() );
        ct.load( new CharOCapUmlaut() );
        ct.load( new CharUAcute() );
        ct.load( new CharUCircumflex() );
        ct.load( new CharUGrave() );
        ct.load( new CharUUmlaut() );
        ct.load( new CharUCapAcute() );
        ct.load( new CharUCapCircumflex() );
        ct.load( new CharUCapGrave() );
        ct.load( new CharUCapUmlaut() );

        ct.load( new CharCopyright() );
        ct.load( new CharExclamInverted() );
        ct.load( new CharQuestionMarkInverted() );
        ct.load( new CharQuoteDoubleLeft() );
        ct.load( new CharQuoteDoubleRight() );
        ct.load( new CharQuoteSingleLeft() );
        ct.load( new CharQuoteSingleRight() );
        ct.load( new CharRegisteredTrademark() );
        ct.load( new CharSSharp() );
    }
}
