/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-08 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus.pdf;

import com.lowagie.text.pdf.*;
import com.lowagie.text.Document;
import platypus.*;

/**
 * Performs non-content actions related to the start of a new page. Per iText,
 * new-page content should be added using the onEndPage() method.
 *
 * Reference:  http://itextdocs.lowagie.com/tutorial/directcontent/pageevents/index.php
 *
 * @author alb
 */
public class OnPageStart extends PdfPageEventHelper
{
    private GDD gdd;

    OnPageStart()
    {
        gdd = GDD.getInstance();
    }
    
    /**
     * Perform all new-page related functions that don't involve placing content on the page.
     *
     * Note: It appears that closing the iText document generates a call to onStartPage. To avoid
     * incrementing page counts incorrectly, we test for the outfile being open or both outfile
     * closed and no text processed, which means we're at the beginning of the first page just
     * before we open the output file and write out the first text.
     *
     * @param writer PdfWriter (supplied by iText)
     * @param doc PdfWriter (supplied by iText)
     */
    @Override
    public void onStartPage( final PdfWriter writer, final Document doc )
    {
        if ( gdd.getOutfile().isOpen() ||
                ( ! gdd.getOutfile().isOpen() && ! gdd.getOutfile().hasTextBeenProcessed() ))
        {
            incrementPageCounter();
            incrementPageNumber();
            putFormattedNumberInSystemStrings();
        }
    }

    public void putFormattedNumberInSystemStrings()
    {
        PageNumberFormat pnf =  gdd.getPageNumberFormat();
        String formattedNumber = pnf.formatPageNumber( gdd.getPageNumber().get() );

        if ( formattedNumber != null ) {
            Platypus.platyStrings.add( "$_formatted_page_number", formattedNumber );
        }
    }
    /**
     * Increment the page count. This is not the same as the page number, which is
     * the number actually printed on the page. This is just a count of which page
     * in the final document we are on.
     */
    public void incrementPageCounter()
    {
        gdd.getPageCounter().increment();
    }

    /**
     * Increment the page number. This is the number actually printed on the page; it might
     * differ from the page count.
     */
    public void incrementPageNumber()
    {
        gdd.getPageNumber().increment();
    }
}
