/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-08 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus.pdf;

import platypus.*;
import com.lowagie.text.*;
import com.lowagie.text.pdf.*;

import java.io.FileNotFoundException;
import java.io.FileOutputStream;

/**
 * Handles document creation/termination and command routines in iText library
 * @author alb
 */
public class OutfilePdf extends platypus.Outfile
{
    //========= iText constructs (identified by the iT prefix)

    private Document iTDocument;

    private PdfWriter iTWriter;

    private Paragraph iTPara = null;

    /** for writing to columns */
    private PdfContentByte iTContentByte;

    /** a column of text */
    private ColumnText iTColumn;

    //======== Other variables
    private final GDD gdd;
    private final Literals lits;

    public OutfilePdf()
    {
        gdd = GDD.getInstance();
        lits = Literals.getInstance();
        setOpenStatus( false );
        gdd.setColumns( new Columns() );
    }

    /**
     * creates and opens iText document into which we write the ouput.
     *
     * Important note: This method is called only when Platypus has text
     * to emit, so we know something about state at this point: All commands
     * that must precede the first text output have been processed.
     *
     * @return  Status.OK on normal completion; otherwise, an error code.
     */
    private int open()
    {
        final Rectangle pageSize =
                new Rectangle( gdd.getPageSize().getWidth(),
                               gdd.getPageSize().getHeight() );

        iTDocument = new Document( pageSize,
                gdd.getMargin().getSize( Margins.LEFT ),
                gdd.getMargin().getSize( Margins.RIGHT ),
                gdd.getMargin().getSize( Margins.TOP ),
                gdd.getMargin().getSize( Margins.BOTTOM ));


        FileOutputStream fos;
        try
        {
            fos = new FileOutputStream( getFilename() );
            iTWriter = PdfWriter.getInstance( iTDocument, fos );
        }
        catch (DocumentException e)
        {
            return( Status.IO_ERR );
        }
        catch( FileNotFoundException e)
        {
            return( Status.FILE_NOT_FOUND_ERR );
        }

        // add the routines for iText to call automatically
        iTWriter.setPageEvent( new OnPageStart() );
        iTWriter.setPageEvent( new OnPageEnd() );

        // set up the needed iText fields in the footer
        PageFooterPdf pfpdf = (PageFooterPdf) gdd.getFooter();
        pfpdf.setDocument( iTDocument );
        pfpdf.setWriter( iTWriter );

        // add metadata referring to Platypus
        PlatyVersion ver = new PlatyVersion();
        iTDocument.addCreator( lits.getLit( "PLATYPUS_CREATED_BY") + ver.toString() + "  " +
                lits.getLit( "AVAILABLE_AT_PZ_ORG" ));

        // open the document
        iTDocument.open();

        // are the margins mirrored?
        if ( gdd.getMargin().areMarginsMirrored() ) {
            iTDocument.setMarginMirroring( true );
        }

        // get the ContentByte for writing to the columns
        iTContentByte = iTWriter.getDirectContent();

        iTColumn = new ColumnText( iTContentByte );

        return( Status.OK );
    }

    /**
     * Adds paragraph to document. The paragraph we are adding is always
     * the current one pointed to by iTPara. If it's null, there is nothing
     * to add, so we just return.
     *
     * @return Status.OK if all goes well, otherwise Status.ERR or Status.INVALID_PARAM
     */
    @Override
    public int addParagraph()
    {
        int ret = Status.OK;

        if( iTPara == null )
            return( ret );

        switch( gdd.getAlignment().getPresent() )
        {
            case CENTER:
                iTPara.setAlignment( Element.ALIGN_CENTER );
                break;
            case JUSTIFIED:
                iTPara.setAlignment( Element.ALIGN_JUSTIFIED );
                break;
            case LEFT:
                iTPara.setAlignment( Element.ALIGN_LEFT );
                break;
            case RIGHT:
                iTPara.setAlignment( Element.ALIGN_RIGHT );
                break;
            default:
                gdd.getLogger().warning( lits.getLit( "ERR_INVALID_TEXT_ALIGNMENT" ));
                iTPara.setAlignment( Element.ALIGN_LEFT );
                gdd.getAlignment().setPresent( Alignment.type.LEFT );
                ret = Status.INVALID_DATA;
                break;
        }

        iTPara.setSpacingBefore( gdd.getParaSkip().getAmount() );

        try
        {
            iTColumn.addElement( iTPara );
        }
        // theoretically impossible as long as we're adding paragraphs
        catch( IllegalArgumentException e )
        {
           gdd.getLogger().warning( "Illegeal Argument Exception:" +
                                lits.getLit( "LINE_NUM_SIGN" ) + " " +
                                GID.getInstance().getCurrLine().getLineNumber() );
            System.err.println( "iTPara is: " + iTPara );
            System.err.println( e );
            ret =  Status.INVALID_PARAM;
        }
        catch( NullPointerException e )
        {
            gdd.getLogger().warning( "Null pointer exception:" +
                                lits.getLit( "LINE_NUM_SIGN" ) + " " +
                                GID.getInstance().getCurrLine().getLineNumber() );
            System.err.println( "iTPara is: " + iTPara );
            System.err.println( e );
            ret =  Status.ERR;
        }
        finally
        {
            // set iTPara to null so that next write creates a new paragraph
            iTPara = null;
        }

        return( ret );
    }

    /**
     * Creates a new page in the document.
     *
     * @return Status.OK if all went well, Status.DOCUMENT_CLOSED_ERR if Document not open
     */
    @Override
    public int newPage()
    {
        int ret = Status.OK;
        boolean wasPageAdded;

        // make sure iText does not ignore multiple new pages in a row (its default is to ignore)
        iTWriter.setPageEmpty( false );

        // if we are in a paragraph we must add it first.
        if ( iTPara != null ) {
            addParagraph();
        }
//        else  {
//            iTWriter.setPageEmpty( false );
////            // if not we must add a mythical paragraph due to bug in iText
////            try
////            {
////                iTDocument.add( new Chunk( "" ));
////            }
////            catch( DocumentException e )
////            {
////                ret = Status.DOCUMENT_CLOSED_ERR;
////            }
//        }

        // now write out any unwritten content to the PDF page(s)
        addColumnsContentToDocument();

        // then add the page.
        if (( wasPageAdded = lowLevelPdfNewPage()) ) {
//            // if successful, set columns all columns to start at top of next page
//            gdd.getColumns().startColumnsAtTop();
            // then set our location to first column
            gdd.setCurrentColumn( 0 );
        }
        else {
            ret = Status.DOCUMENT_CLOSED_ERR;
        }

        if ( ret != Status.OK ) {
            return( ret );
        }
        else {
            return(Status.PAGE_ADDED );
        }
    }

    @Override
    public boolean lowLevelPdfNewPage()
    {
        boolean wasPageAdded;

        try
        {
            iTWriter.setPageEmpty( false );           // forces the page to be emitted.
            wasPageAdded = iTDocument.newPage();
        }
        catch( Exception e )
        {
            wasPageAdded = false;
        }
        return( wasPageAdded );
    }

    /**
     * Add a new paragraph to an iText document. If the current paragraph has
     * already been flushed to the document (so iTPara = null ), we write out
     * a single space and then flush it as the new paragraph to the PDF file.
     */
    @Override
    public void doNewParagraph()
    {
        if ( iTPara == null ) {
            emitText( " " );
        }
        addParagraph();
    }

    /**
     * create a new paragraph in iText and indent the first line as specified.
     */
    private void newParagraph()
    {
        iTPara = new Paragraph( gdd.getLeading().getAmount() );

        // noIndent applies only to one paragraph. So, test for it here. If it's
        // set (that is, true/on) for this paragraph, then reset it to false for
        // the next paragraph and skip any indenting operation.
        if( Boolean.valueOf( gdd.getNoIndent().getValue() ) == Status.ON ) {
            gdd.getNoIndent().setValue( false );
        }
        else {
            indentParagraphFirstLine( gdd.getParaIndent().getAmount() );
        }
    }

    /**
     * Indent the first line of the paragraph by indentPoints
     *
     * This code requires explanation. iText, prior to v. 2, did not have a way
     * to indent the first line of a paragraph. So, what we do is add spaces to make
     * up the intended indent. The space character in the 7-point Helvetica font is
     * 1.946 pts wide. We divide the specified indent by this amount. then round, and
     * we get the number of spaces to add to the paragraph. Luckily, iText does not
     * compress sequential spaces, so this actually works. I admit, it's a hack. -alb
     *
     * @param indentPoints  amount to indent the first line
     */
    private void indentParagraphFirstLine( final float indentPoints )
    {
        if ( indentPoints >  0 ) {
            final Font font7H = new Font( Font.HELVETICA, 7, Font.NORMAL );
            final Chunk indent2pt = new Chunk( " ", font7H );

            final float reps = gdd.getParaIndent().getAmount() / 1.946f;
            for ( int i = 0; i < (int)Math.floor( reps + 0.5f ); i++ )
                iTPara.add( indent2pt );
            iTPara.add( indent2pt );
        }
    }

    /**
     * iText-required function at end of processing. Basically, output any not-yet-output
     * content to the document. Then, close the file.
     */
    @Override
    public void close()
    {
        // First output any unwritten paragraph text to the column(s)
        if ( iTPara != null ) {
            addParagraph();
        }

        // then output the column content
        addColumnsContentToDocument();

        // now, we can close (note: following steps must occur in this order)
        setOpenStatus( false );
        iTDocument.close();
    }

    /**
     * Adds any material in the columns to the document. This is where the content is
     * written to the PDF file.
     */
    @Override
    public void addColumnsContentToDocument()
    {
        int status = ColumnText.NO_MORE_COLUMN;

        // for temporary debug purposes
        // System.out.println( "y-position: " + getCurrLocationY() );  // top = 0

        while( ColumnText.hasMoreText( status ))
        {
            if ( gdd.getCurrentColumn() >= gdd.getColumns().getCount() ) {
                iTDocument.newPage();
                gdd.setCurrentColumn( 0 );
            }

            final int currColNumber = gdd.getCurrentColumn();

            /** the two right/left edges of the current column */
            float leftEdge, rightEdge;

            /** the top and bottom edges of the current column */
            float topEdge, bottomEdge;

            leftEdge = gdd.getMargin().getSize( Margins.LEFT );

            for( int i = 0; i < gdd.getCurrentColumn(); i++ )
            {
                leftEdge += gdd.getColumns().getColumn( i ).getWidth() +
                           gdd.getColumns().getColumn( i ).getGutter();
            }

            rightEdge  = leftEdge + gdd.getColumns().getColumn( currColNumber ).getWidth();
            topEdge    = gdd.getPageSize().getHeight() - gdd.getMargin().getSize( Margins.TOP ) -
                         gdd.getColumns().getColumn( currColNumber ).getVertSkip();

            // bottom edge is the lesser of: top edge - column height or top edge - bottom margin.
            bottomEdge = Math.min(
                    topEdge - gdd.getColumns().getColumn( currColNumber ).getHeight(),
                    topEdge - gdd.getMargin().getSize( Margins.BOTTOM ));

            // now set the column size

            iTColumn.setSimpleColumn(
                    leftEdge,
                    bottomEdge,
                    rightEdge,
                    topEdge );
            
            try
            {
                status = iTColumn.go();
                if ( ColumnText.hasMoreText( status ) ) {
                    gdd.setCurrentColumn( gdd.getCurrentColumn() + 1 );
                }
            }
            catch ( DocumentException de ) {        //TODO: Replace with true exception handling
                System.out.println( "Exception adding text to iTColumn (ColumnText) in OutfilePdf" );
                System.out.println( "Occurred at line: " +
                GID.getInstance().getCurrLine().getLineNumber() );
            }
        } // end of while 

    }

    /**
     * Two actions can result from an item in an input file: it can set the value of a
     * variable (such as margins, for example) or result in text that must be emitted to the
     * output file stream. emit() handles this latter function.
     *
     * @param s text string to send to output
     */
    @Override
    public void emitText( String s )
    {
        Literals lits = Literals.getInstance();

        if ( s == null ) {
            return; //TODO: log an error here
        }

        if ( s.length() == 0 ) {
            return; // TODO: log a warning here
        }

        // at this point, we know there's something to output. So, first we
        // make sure output file is open, then we emit the token. The first
        // time through, the file will be closed, and we'll open it from here.
        // This prevents us creating a file until we know for sure there is
        // something to write out.

        if ( ! isOpen() ) {
            if ( open() != Status.NO_ERROR ) {
                gdd.getLogger().severe( lits.getLit( "ERR_OUTFILE_OPEN" ) + " " +
                                   getFilename() + " " +
                                   lits.getLit( "ERR_FATAL_EXITING" ));
                System.exit( Status.FATAL_ERR );
            }
            setOpenStatus( true );
        }

        // check if we are in an existing paragraph. If not, create a new one.
        if ( iTPara == null ) {
            newParagraph();
        }
        
        // now create a Chunk containing the text in String s
        final Chunk chunk = new Chunk( s, gdd.getCurrFont().getFont() );

        // if strikethrough is on, then set it here for this chunk
        if ( gdd.getStrikethru().isActive() ) {
            chunk.setUnderline( PlatyConst.DEFAULT_STRIKETHRU_LINE_WIDTH, 
                                gdd.getStrikethru().getAmount() );
        }

        // if underline is on, then set it for this chunk
//        if ( gdd.getUnderlineSwitch() == Status.ON ) {
//            chunk.setUnderline( PlatyConst.DEFAULT_UNDERLINE_THICKNESS,
//                                gdd.getUnderlineHeight() );
//        }
        if ( gdd.getUnderline().isInEffect() ) {
            chunk.setUnderline( PlatyConst.DEFAULT_UNDERLINE_THICKNESS,
                                gdd.getUnderline().getLocation() );
        }

        // before writing the Chunk to the Paragraph, first check to see whether
        // a change in leading has occurred. If it has, the Chunk must be added
        // first to a phrase and the phrase must be added to the Paragraph. This
        // is due to the fact that Chunks cannot specify leading. Phrases are the
        // lowest-level iText element that can specify leading.

        if ( gdd.getLeading().isLeadingChanged() ) {
            final Phrase ph = new Phrase( gdd.getLeading().getAmount(), chunk );
            iTPara.add( ph );
            gdd.getLeading().setLeadingChangedSwitch( false );
        }
        else {
            iTPara.add( chunk );
        }

        textProcessedFlag = true;    // show that we have processed some text.
    }

    /**
     * return the our Y (or vertical) position in the column. Note re coordinates:
     * the PDF page starts with a 0,0 coordinate in the lower left corner.
     *
     * @return  the current Y location in the current column, using points.
     */
    public float getCurrLocationY()
    {
        return( iTColumn.getYLine() );
    }
}
