/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-08 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus.pdf.commands;

import platypus.*;
import platypus.pdf.OutfilePdf;

/**
 * Executes command for: specifying a change in the number of columns.
 * The default number of columns is 1.
 *
 * @author alb
 */
public class CommandColumns extends Command
{
    public CommandColumns()
    {
        setAbbrev( "[columns:" );
        setName( lits.getLit( "DO_COLUMNS" ));
        setType( TokTyp.COLUMNS );
        setRoot( "column" );
    }

    /**
     * Changes the number of columns. For the time being (this will change),
     * modifying column count results in the new columns starting on the next page
     *
     * @param actualCommand the actualCommand from the input document
     * @return Status.OK, if all went well. Otherwise the appropriate error code.
     */
    @Override
    public int process( final String actualCommand )
    {
        if ( actualCommand == null ) {
            return( Status.INVALID_PARAM_NULL );
        }

        if ( actualCommand.length() < ( getAbbrev().length() + 1 ) ||
            ! actualCommand.startsWith( getAbbrev() )) {
            return( Status.INVALID_COMMAND );
        }

        if ( gdd == null ) {
            return( Status.INVALID_GDD );
        }

        // how many columns were specified?
        ParsedUnit columns =
                Parser.getUnitValue( actualCommand.substring( getAbbrev().length(),
                        actualCommand.length() ));

        if ( columns.getUnit() == PlatyConst.unit.ERROR || columns.getAmount() < 1.0f ) {
            return( Status.INVALID_PARAM );
        }

        // if new column count is different then existing column count, flush any unwritten
        // text, and then set up the new columns.
        if ( gdd.getColumns().getCount() != columns.getAmount() ) {
            flushExistingText();
            setupNewColumns( (int) columns.getAmount() );
//            // then skip a page (for the nonce) --- will remove this logic shortly
//            ///gdd.outfile.iTDocument.newPage();
//            gdd.getOutfile().lowLevelPdfNewPage();
//            gdd.setCurrentColumn( 0 );

//            // then set up the new columns
//            gdd.setColumns( new Columns( (int) columns.getAmount() ));
        }        
        return( Status.OK );
    }

    /**
     * Set up the new columns starting wherever we are on the page.
     * We first figure where we are vertically on the page, and then
     * set up the columns to occupy the rest of the vertical space on the page
     *
     * @param newColumnCount number of columns to set up
     */
    private void setupNewColumns( int newColumnCount )
    { 
        // get our current Y position
        OutfilePdf opdf = (OutfilePdf) gdd.getOutfile();
        final float currY = opdf.getCurrLocationY();

        final float vertSkip = computeVerticalSkip( currY );

        // if we are going from a 1-col format to multiple columns, then
        // we do it right where we are.
        if ( gdd.getColumns().getCount() == 1 ) {
            gdd.setColumns( new Columns( newColumnCount, vertSkip ));
        }
        else
        if ( newColumnCount == 1 ) {
        // if the new column count is 1, then we can close off where we are on the
        // page and go back to a single column. If we are in the leftmost column,
        // then we make the conversion right there on the page. If we are not in the
        // leftmost column, then we presume that the leftmost column has been filled
        // to the bottom of the page, so we need to do a page eject and start on the
        // next page.

            adjustCurrentColumn();
            // are we in the leftmost column?
            if ( gdd.getCurrentColumn() == 0 ) {
                gdd.setColumns( new Columns( 1, vertSkip ));
            }
            else {
                // if we're not in the first column, we have to assume
                // that the leftmost column is full to the bottom of the
                // page, so we push out a new page and start in the left
                // column of that page (column 0)
                gdd.getOutfile().lowLevelPdfNewPage();
                gdd.setColumns( new Columns( 1, 0f ));
                gdd.setCurrentColumn( 0 );
            }
        }
        else
        // we are going from some number of columns > 1 to another number > 1
        // so start a new page (for the time being) //TODO: resolve the many-to-many column change
        {
            gdd.getOutfile().lowLevelPdfNewPage();
            gdd.setColumns( new Columns( newColumnCount, 0f ));
            gdd.setCurrentColumn( 0 );
        }
    }

    /**
     * When flushExistingText() is called in process(), it bumps up the column number, due
     * to the way the loop works. Here we change the current column back. This might need to
     * be refactored away.
     */
    public void adjustCurrentColumn()
    {
        final int currentCol = gdd.getCurrentColumn();
        if ( currentCol > 0 ) {
            gdd.setCurrentColumn( currentCol - 1 );
        }
    }

    /**
     * Computes how much vertical distance there is between the current baseline and
     * the top margin. Recall that in PDFs the Y values increase from the bottom upward.
     * So, the top edge of the paper (the page height) is the highest possible value.
     * From this we subtract the top margin and the current Y value from the baseline.
     * This gives us how much distance exists between the top margin and the current baseline.
     *
     * @param currYposition y-position of current baseline
     * @return the vertical skip between top margin and current baseline
     */
    public float computeVerticalSkip( final float currYposition )
    {
        return( gdd.getPageSize().getHeight() - gdd.getMargin().getSize( Margins.TOP ) -
                currYposition );
    }

    /**
     * write out any text that has not yet been written out
     */
    private void flushExistingText()
    {
        gdd.getOutfile().addColumnsContentToDocument();
    }
}
