/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-07 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus.pdf.commands;

import platypus.*;

/**
 * Executes command for: specifying a new typeface.
 *
 * @author alb
 */
public class CommandFontFace extends Command
{
    public CommandFontFace()
    {
        setAbbrev( "[ff:" );
        setName( lits.getLit( "DO_FONT_FACE" ));
        setType( TokTyp.FONT_FACE );
    }

    /**
     * Changes the current font to a new typeface. (Size and other features are unchanged)
     *
     * @param actualCommand the actualCommand (containing, in part, the name of the new typeface)
     * @return Status.OK, if all went well. Otherwise the appropriate error code.
     */
    @Override
    public int process( final String actualCommand )
    {

        if ( actualCommand == null ) {
            return( Status.INVALID_PARAM_NULL );
        }

        if ( actualCommand.length() < ( getAbbrev().length() + 1 ) ||
            ! actualCommand.startsWith( getAbbrev() )) {
            return( Status.INVALID_COMMAND );
        }

        if ( gdd == null ) {
            return( Status.INVALID_GDD );
        }

        final String commandPrefix = getAbbrev();
        final String commandSuffix = "]";

        // the start and end of the commandString that contains the font name
        int start = -1;
        int end = -1;

        if ( actualCommand.startsWith( commandPrefix )) {
            start += commandPrefix.length();
        }

        if ( actualCommand.endsWith( commandSuffix )) {
            end = actualCommand.length() - commandSuffix.length();
        }

        if ( start == -1 || end == -1 ) {
            return( Status.INVALID_PARAM );
        }

        final String rawFontName = actualCommand.substring( start + 1, end );

        final String fontName = Parser.extractParameter( rawFontName );
        if ( fontName == null ) {
            logError( fontName );
            return( Status.INVALID_PARAM );
        }

        final PlatyFont newFont = new PlatyFont( fontName );
        if ( newFont.getFont() == null ) {
            logError( fontName );
            return( Status.INVALID_PARAM_NULL );
        }
        else {
            gdd.getLogger().fine( lits.getLit( "LINE_NUM_SIGN" ) + " " + newFont.getLineNumber() +
                    ": " + lits.getLit( "TYPEFACE_SET_COLON" ) + " " + fontName + " " +
                    newFont.getSize() + " " + lits.getLit( "POINTS" ));
        }

        gdd.setCurrFont( newFont );
        return( Status.OK );
    }

    /**
     * log error message if invalid font name is specified
     *
     * @param fontName the invalid font name
     */
    private void logError( final String fontName )
    {
        gdd.getLogger().warning( lits.getLit( "ERR_INVALID_FONT") + " " +
                fontName + " at " + gid.getCurrLine().getLineNumber() +
                ". " + lits.getLit( "IGNORED" ));
    }
}

// from CommandPdf class:
//
//    /**
//     * Changes the current font to a new typeface. (Size and other features are unchanged)
//     * @param commandString the command string containing the name of the new typeface
//     * @return Status.OK, if all went well. Otherwise the appropriate error code.
//     */
//    @Override
//    protected int doFontFace( final String commandString )
//    {
//        final String commandPrefix = "[ff:";
//        final String commandSuffix = "]";
//
//        // the start and end of the commandString that contains the font name
//        int start = -1;
//        int end = -1;
//
//        if ( commandString == null ) {
//            return( Status.INVALID_PARAM_NULL );
//        }
//
//        if ( commandString.startsWith( commandPrefix )) {
//            start += commandPrefix.length();
//        }
//
//        if ( commandString.endsWith( commandSuffix )) {
//            end = commandString.length() - commandSuffix.length();
//        }
//
//        if ( start == -1 || end == -1 ) {
//            return( Status.INVALID_PARAM );
//        }
//
//        final String fontName = commandString.substring( start + 1, end );
//
//        final PlatyFont newFont = new PlatyFont( fontName );
//        if ( newFont.getFont() == null )
//        {
//            gdd.getLogger().warning( lits.getLit( "ERR_INVALID_FONT") + " " +
//                    fontName + " at " + gid.getCurrLine().getLineNumber() +
//                    ". " + lits.getLit( "IGNORED" ));
//
//            return( Status.INVALID_PARAM_NULL );
//        }
//        else
//        {
//            gdd.getLogger().fine( lits.getLit( "LINE_NUM_SIGN" ) + " " + newFont.getLineNumber() +
//                    ": " + lits.getLit( "TYPEFACE_SET_COLON" ) + " " + fontName + " " +
//                    newFont.getSize() + " " + lits.getLit( "POINTS" ));
//        }
//
//        gdd.setCurrFont( newFont );
//        return( Status.OK );
//    }
//