/*-
 *  (c) Copyright 2006-7 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 *
 *  For more information about platter, go to http://platypus.pz.org
 */

package platypus.pdf.commands;

import platypus.*;

/**
 * Executes command for: specifying custom font height
 *
 * @author alb
 */
public class CommandFontSize extends Command
{
    public CommandFontSize()
    {
        setAbbrev( "[fsize:" );
        setName( lits.getLit( "DO_FONT_SIZE" ));
        setType( TokTyp.FONT_SIZE );
    }

    /**
     *
     * @param actualCommand the command as found in the input Platypus file
     * @return status Status.OK; Status.INVALID_GDD if gdd has not been initialized;
     *          Status.INVALID_PARAM for a mis-specified height, Status.ERR_FONT_SIZE
     *          if the height is correctly specified but outside the allowed range.
     *          Status.INVALID_COMMAND if the font root is badly formed.
     */
    @Override
    public int process( final String actualCommand )
    {

        if ( actualCommand == null ) {
            return( Status.INVALID_PARAM_NULL );
        }

        if ( gdd == null ) {
            return( Status.INVALID_GDD );
        }

        // verify that the passed-in actual command is syntactically valid:
        // the command should contain the abbreviated form of the command and, since there is
        // a parameter, it should also be longer than this abbreviated form.

        if ( actualCommand.indexOf( getAbbrev() ) == -1 ||
                actualCommand.length() < getAbbrev().length() + 1 ) {
            return( Status.INVALID_COMMAND );
        }

        // the start and end of the actualCoommand that contains the font size
        int start = -1;
        int end = -1;
        final String suffix = "]";

        if ( actualCommand.startsWith( getAbbrev() )) {
            start += getAbbrev().length();
        }

        if ( actualCommand.endsWith( suffix )) {
            end = actualCommand.length();
        }

        if ( start == -1 || end == -1 ) {
            return( Status.INVALID_COMMAND );
        }

        final String fontSize = actualCommand.substring( start + 1, end );
        final FontSize fs = new FontSize( fontSize, actualCommand );

        // fs.amount (the new size) must be > PlatyConst.MINIMUM_FONT_SIZE and
        // must not equal FLOAT.MIN_VALUE, which would signal an error in the
        // amount field.
        if ( fs.getAmount() != Float.MIN_VALUE && fs.getAmount() > PlatyConst.MINIMUM_FONT_SIZE ) {
            final PlatyFont newFont = new PlatyFont( fs.getAmount() );
            if ( newFont.getFont() == null ) {
                return( Status.INVALID_PARAM );
            }
            gdd.setCurrFont( newFont );

            // change the leading to reflect the size of the new font
            // and set a flag indicating this, so that OutfilePdf can
            // handle it correctly.
            gdd.getLeading().setAmount( PlatyConst.LEADING_TO_FONT_RATIO * fs.getAmount() );

            return( Status.OK );
        }
        else {
            // if the number < PlatyConst.MINIMUM_FONT_SIZE, state error
            if ( fs.getAmount() != Float.MIN_VALUE ) {
                gdd.getLogger().warning( lits.getLit( "ERR_FONT_SIZE_TOO_SMALL" ) +
                        " " + actualCommand  + " " + lits.getLit( "AT_LINE") +
                        ": " + fs.getLineNumber() );
                return( Status.ERR_FONT_SIZE );
            }

            return( Status.INVALID_PARAM );
        }

    }
}

// original code from CommandPDF
//
//         final String commandPrefix = "[fsize:";
//         final String commandSuffix = "]";
//
//         // the start and end of the commandString that contains the font size
//         int start = -1;
//         int end = -1;
//
//         if ( commandString == null ) {
//             return( Status.INVALID_PARAM_NULL );
//         }
//
//         if ( commandString.startsWith( commandPrefix )) {
//             start += commandPrefix.length();
//         }
//
//         if ( commandString.endsWith( commandSuffix )) {
//             end = commandString.length();
//         }
//
//         if ( start == -1 || end == -1 ) {
//             return( Status.INVALID_PARAM );
//         }
//
//         final String fontSize = commandString.substring( start + 1, end );
//         final FontSize fs = new FontSize( fontSize, commandString );
//
//         // fs.amount (the new size) must be > PlatyConst.MINIMUM_FONT_SIZE and
//         // must not equal FLOAT.MIN_VALUE, which would signal an error in the
//         // amount field.
//         if ( fs.getAmount() != Float.MIN_VALUE && fs.getAmount() > PlatyConst.MINIMUM_FONT_SIZE ) {
//             final PlatyFont newFont = new PlatyFont( fs.getAmount() );
//             if ( newFont.getFont() == null ) {
//                 return( Status.INVALID_PARAM );
//             }
//             gdd.setCurrFont( newFont );
//
//             // change the leading to reflect the size of the new font
//             // and set a flag indicating this, so that OutfilePdf can
//             // handle it correctly.
//             gdd.getLeading().setAmount( PlatyConst.LEADING_TO_FONT_RATIO * fs.getAmount() );
//
//             return( Status.OK );
//         }
//         else {
//             // if the number < PlatyConst.MINIMUM_FONT_SIZE, state error
//             if ( fs.getAmount() != Float.MIN_VALUE ) {
//                 gdd.getLogger().warning( lits.getLit( "ERR_FONT_SIZE_TOO_SMALL" ) +
//                      " " + commandString  + " " + lits.getLit( "AT_LINE") +
//                      ": " + fs.getLineNumber() );
//             }
//
//             return( Status.INVALID_PARAM );
//         }
//     }
