/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-07 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus.pdf.commands;

import platypus.Command;
import platypus.TokTyp;
import platypus.Status;

/**
 * Executes command for: turning on mirrored margins
 *
 * Mirrored margins are used when the margins are of different sizes (such as to
 * accommodate the binding of a book) and we want the left and right pages to be
 * mirrors of each other (so that, for example, the wider margin is always the
 * one for the binding).
 *
 * When the margins are of equal size, all pages are printed the same, so there is
 * no concept of mirrored pages.
 *
 * @author alb
 */
public class CommandMarginsMirrored extends Command
{
    public CommandMarginsMirrored()
    {
        setAbbrev( "[+marginmirror]" );
        setName( lits.getLit( "MARGINS_MIRRORED" ));
        setType( TokTyp.MARGIN_MIRROR );
    }

    /**
     * This can only be turned on. Margins can't be changed once text has been output,
     * so there is no corresponding 'off' command.
     *
     * @param actualCommand the command as found in the input Platypus file
     * @return Status.OK or error code depending on the nature of the error.
     */
    public int process( final String actualCommand )
    {
        if ( gdd == null ) {
            return( Status.INVALID_GDD );
        }

        if ( actualCommand == null ) {
            return( Status.INVALID_PARAM_NULL );
        }
        else {
            if ( actualCommand.equals( getAbbrev() )) {
                if ( ! gdd.getOutfile().hasTextBeenProcessed() ) {
                    gdd.getMargin().setMarginsMirrored( true );
                }
                else {
                    // no changes after text has been output
                    return( Status.OP_NOT_PERMITTED );
                }
            }
            else {
                return( Status.INVALID_COMMAND );
            }
        }
        return( Status.OK );
    }
}

//
// from CommandProcessor.java:
//
//    private void doMarginMirror( final boolean onOff )
//    {
//        if ( onOff != Status.ON || gdd.getOutfile().hasTextBeenProcessed() == true  ) {
//            gdd.getLogger().severe( lits.getLit( "ERR_MIRRORED_MARGINS" ) + " " +
//                                lits.getLit( "LINE_NUM_SIGN") +
//                                GID.getInstance().getCurrLine().getLineNumber() );
//        }
//        else {
//            gdd.getMargin().setMarginsMirrored( onOff );
//        }
//    }


