/**
 * (c) Copyright 2007 Pacific Data Works LLC. All Rights Reserved.
 * Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus.pdf.commands;

import platypus.*;
import platypus.PageSize;

/**
 * Executes command for: specifying custom page height
 *
 * @author alb
 */
public class CommandPageHeight extends Command
{
    public CommandPageHeight()
    {
        setAbbrev( "[pageheight:" );
        setName( lits.getLit( "PAGE_HEIGHT" ));
        setType( TokTyp.PAGE_HEIGHT );
    }

    /**
     * Verify the height, check it against the established PDF min/max height
     * and then set the value if all is well.
     *
     * @param actualCommand the command as found in the input Platypus file
     * @return status Status.OK; Status.INVALID_GDD if gdd has not been initialized;
     *          Status.INVALID_PARAM for a mis-specified height, Status.ERR_PAGE_HEIGHT
     *          if the height is correctly specified but outside the allowed range.
     */
    public int process( String actualCommand )
    {
        if ( gdd == null ) {
            return( Status.INVALID_GDD );
        }

        if ( actualCommand == null ) {
            return( Status.INVALID_PARAM );
        }

        try {
            // verify that the passed-in actual command is syntactically valid:
            // the command should contain the abbreviated form of the command and, since there is
            // a parameter, it should also be longer than this abbreviated form.

            if ( actualCommand.indexOf( this.getAbbrev() ) == -1 ||
                    actualCommand.length() < this.getAbbrev().length() + 1 ) {
                return( Status.INVALID_PARAM );
            }

            PageSize psize = gdd.getPageSize();
            if ( psize == null ) {
                return( Status.INVALID_GDD );
            }

            // make sure that the requested page height is within PDF's limits
            // start by parsing the numeric portion and the units and getting
            // the resulting number of points.
            final float PDF_MAX_HEIGHT = 14400f;
            final float PDF_MIN_HEIGHT = 3f;
            
            String height = actualCommand.substring(
                    this.getAbbrev().length(), actualCommand.length() - 1 );

            ParsedUnit pu = Parser.getUnitValue( height );
            if ( pu.getUnit() == PlatyConst.unit.ERROR ) {
                return( Status.INVALID_PARAM );
            }

            float heightInPoints =
                    PlatyConst.convertUnitsToPoints( pu.getUnit(), pu.getAmount() );

            if ( heightInPoints > PDF_MAX_HEIGHT ||
                 heightInPoints < PDF_MIN_HEIGHT ) {
                return( Status.ERR_PAGE_HEIGHT );
            }

            gdd.getPageSize().setHeight( heightInPoints );
        }
        catch ( IllegalArgumentException e ) {
            return( Status.INVALID_PARAM );
        }

        return( Status.OK );
    }
}
