/**
 * (c) Copyright 2007 Pacific Data Works LLC. All Rights Reserved.
 * Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus.pdf.commands;

import platypus.*;
import platypus.PageSize;

/**
 * Executes command for: specifying custom page width
 *
 * @author alb
 */
public class CommandPageWidth extends Command
{
    public CommandPageWidth()
    {
        setAbbrev( "[pagewidth:" );
        setName( lits.getLit( "PAGE_WIDTH" ));
        setType( TokTyp.PAGE_WIDTH );
    }

    /**
     * Call the method in the PageSize class that sets the page width
     *
     * @param actualCommand the command as found in the input Platypus file
     * @return status Status.OK; Status.INVALID_GDD if gdd has not been initialized;
     *          Status.INVALID_PARAM for a mis-specified width, Status.ERR_PAGE_WIDTH
     *          if the width is correctly specified but outside the allowed range.
     */
    public int process( String actualCommand )
    {
        if ( gdd == null ) {
            return( Status.INVALID_GDD );
        }

        if ( actualCommand == null ) {
            return( Status.INVALID_PARAM );
        }

        try {

            // verify that the passed-in actual command is valid:
            // the command should contain the abbreviated form of the command and, since there is
            // a parameter, it should also be longer than this abbreviated form.
            if ( actualCommand.indexOf( this.getAbbrev() ) == -1 ||
                    actualCommand.length() < this.getAbbrev().length() + 1 ) {
                return( Status.INVALID_PARAM );
            }

            PageSize psize = gdd.getPageSize();
            if ( psize == null ) {
                return( Status.INVALID_GDD );
            }

            // make sure that the requested page width is within PDF's limits.
            // start by parsing the numeric portion and the units, and converting
            // the resulting number to points.
            final float PDF_MAX_WIDTH = 14400f;
            final float PDF_MIN_WIDTH = 3f;

            String width = actualCommand.substring(
                    this.getAbbrev().length(), actualCommand.length() - 1 );

            ParsedUnit pu = Parser.getUnitValue( width );
            if ( pu.getUnit() == PlatyConst.unit.ERROR ) {
                return( Status.INVALID_PARAM );
            }

            float widthInPoints =
                    PlatyConst.convertUnitsToPoints( pu.getUnit(), pu.getAmount() );

            if ( widthInPoints > PDF_MAX_WIDTH ||
                 widthInPoints < PDF_MIN_WIDTH ) {
                return( Status.ERR_PAGE_WIDTH );
            }

            gdd.getPageSize().setWidth( widthInPoints );            
        }
        catch ( IllegalArgumentException e ) {
            return( Status.INVALID_PARAM );
        }

        return( Status.OK );
    }
}

