/***
 *  Platypus: Page Layout and Typesetting Software (free at platypus.pz.org)
 *
 *  Platypus is (c) Copyright 2006-07 Pacific Data Works LLC. All Rights Reserved.
 *  Licensed under Apache License 2.0 (http://www.apache.org/licenses/LICENSE-2.0.html)
 */

package platypus.pdf.commands;

import platypus.*;

/**
 * Executes command for: specifying the indent of the first line of succeeding paragraphs.
 * 
 * Eventually, there might be some thought given to a negative value, which would outdent
 * the first line (the so-called hanging first line). But at the moment, this is not supported.
 *
 * Note: this value can be overriden for one paragraph via the [noindent] command.
 * 
 * @author alb
 */
public class CommandParaIndent extends Command
{
    public CommandParaIndent()
    {
        setAbbrev( "[paraindent:" );
        setName( lits.getLit( "PARA_INDENT_1ST_LINE" ));
        setType( TokTyp.PARA_INDENT_1 );
    }

    /**
     * Configures the distance to indent the first line of succeeding paragraphs.
     * Default is 0.
     *
     * @param actualCommand the command as found in the input Platypus file
     * @return status Status.OK; or another value that identified the type of error.
     */
    public int process( final String actualCommand )
    {

        if ( actualCommand == null ) {
            return( Status.INVALID_PARAM_NULL );
        }

        if ( actualCommand.length() < ( getAbbrev().length() + 2 ) ||
            ! actualCommand.startsWith( getAbbrev() )) {
            return( Status.INVALID_COMMAND );
        }

        if ( gdd == null ) {
            return( Status.INVALID_GDD );
        }

        // Create a new ParaIndent object, and if valid, replace the one in GDD with it.

        ParaIndent newParaIndent =
                    new ParaIndent( actualCommand.substring( getAbbrev().length() ),
                                      actualCommand.substring( 1,7 ));

        if ( newParaIndent.getAmount() == Float.MIN_VALUE || newParaIndent.getAmount() <  0 ) {
            newParaIndent.stateError( actualCommand );
            return( Status.INVALID_PARAM );
        }
        else {
            gdd.setParaIndent( newParaIndent );
        }
        return( Status.OK );
    }
}